# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996
#

build_agnt_sed_script () {
	local type=$1
	local addr=$2
	shift 2
# This seemed to work: sed "s/ARS.host.*=.*/ARS.host = address/" For the ARS
# This seemed to work for the .Host entries: sed "s/.Host.*=.*/.Host=xxxx/"


	local sed_s=
	local prop
	
	for prop in $*
	do
        case $type in
        1)
            sed_s="s/$prop.*=.*/$prop = $addr/;$sed_s"       
        ;;
        2)
            sed_s="s/$prop.*=.*/$prop=$addr/;$sed_s"       
        ;;
        esac
	done
	#debug "sed script is:$sed_s"
	SED_SCRIPT=$sed_s
	debug "SED_SCRIPT is:$SED_SCRIPT"
}



# This is a common routine to update property files
# Parameters:
#   Directory
#   File
#   SED script
updt_prop_file () {  
    local src_dir=$1
    local filename=$2
    local sed_scr=$3
    
    local ffile=$src_dir/$filename    
    local bkupfile=$src_dir/$filename.orig
            
    # Make a backup copy of the file if not there already
    mk_bkup_file_if_not_already $ffile $bkupfile
    
    perl -pi -e "$sed_scr" $ffile
        
}

set_am_mnic_on () {
    ffile=$INSTALL_DIR/lwi/runtime/agentmanager/eclipse/plugins/com.ibm.tivoli.cas.manager/WEB-INF/classes/resources/AgentManager.properties
    perl -pi -e "s/^ARS.mnic.*=.*/ARS.mnic = on/g" $ffile
}
set_am_mnic_off () {
    ffile=$INSTALL_DIR/lwi/runtime/agentmanager/eclipse/plugins/com.ibm.tivoli.cas.manager/WEB-INF/classes/resources/AgentManager.properties
    perl -pi -e "s/^ARS.mnic.*=.*/ARS.mnic = off/g" $ffile
}


# Change the Agent Manager manager.properties file
updt_am_mgr_prop () {
	
    local props="CatalogueService.Host \
CertManagement.Host \
AgentQuery.Host \
Registration.Server.Host \
TrustedCertificateQuery.Host \
AgentManagerQuery.Host"

    local src_dir=$INSTALL_DIR/data/AMData
    local filename=manager.properties
    
    local ffile
    local props

    if [ ! -d $src_dir ]; then
    	# If we can't find the source directory, just return.
    	return
    fi

    local foundfiles=`find $src_dir -name "$filename"`

    
    debug "updt_am_mgr_prop"
    # Build the sed commands for all these props
    if [ -z "$1" ]; then
        build_agnt_sed_script 2 $agent_manager_address $props
    else
        build_agnt_sed_script 2 $1 $props
    fi
    
    local sed_scr=$SED_SCRIPT
    for ffile in $foundfiles
    do
    	local fdir=$(dirname $ffile)
    	local fbase=$(basename $ffile)
    	
    	updt_prop_file $fdir $fbase "$sed_scr" 
    	
    done
	
}

# Update the Agent Manager's AgentManager.properties file
updt_am_agentmgr_prop () {
#/opt/ibm/director/lwi/runtime/agentmanager/eclipse/plugins/com.ibm.tivoli.cas.manager/WEB-INF/classes/resources
#AgentManager.properties
# ARS.host entry

    local props="ARS.host"
    
    local src_dir=$INSTALL_DIR/lwi/runtime/agentmanager/eclipse/plugins/com.ibm.tivoli.cas.manager/WEB-INF/classes/resources
    local filename=AgentManager.properties
    local ffile=$src_dir/$filename
    
    debug "updt_am_agentmgr_prop"
    if [ -f $ffile ]; then
        # Build the sed commands for all these props
        build_agnt_sed_script 1 $agent_manager_address $props
        local sed_scr=$SED_SCRIPT
                
        updt_prop_file $src_dir $filename "$sed_scr"
        set_am_mnic_off
        
    else
        debug "AgentManager file not found: $ffile"
        local file_name_parm=AgentManager.properties
        # TRANSLATORS: This error message means the file specified in the $file_name_parm was 
        # not found on the local file system
        sm_msg "`eval_gettext \"File not found: \\\$file_name_parm\"`"
    fi
	
}

find_static_address_for_floater () {
	local floater=$1
	    
    local netalias=`grep $floater $SMHA_CFG_FILE | grep network-alias`
    
    # Get the ip version
    local isIPv6=`echo $netalias | grep alias6`
    local ipversion=4
    if [ -n "$isIPv6" ]; then
        ipversion=6
    fi
    
    # Get the nic
    local nic=${netalias##*,}
    debug "nic: $nic ipversion: $ipversion"
    
    local addrline=`LC_ALL=C /usr/smrshbin/lsnetcfg -n | grep ipv${ipversion}addr_${nic}`    
    debug "addrline:$addrline"
    local addr=${addrline##* }
    debug "addr:$addr"
    # if this is ipv6 then remove the mask
    if [ -n "$isIPv6" ]; then
        addr=${addr%%/*}
    fi
	debug "Return static addr: $addr for floater:$floater"
	RETURN_STATIC_ADDR=$addr
}

replace_agentmgr_ARShost_addr () {
	local addr=${1}

    local src_dir=$INSTALL_DIR/lwi/runtime/agentmanager/eclipse/plugins/com.ibm.tivoli.cas.manager/WEB-INF/classes/resources
    local filename=AgentManager.properties
    local ffile=$src_dir/$filename
                
    debug "replace ARS.host address with $addr"
    
    perl -pi -e "s/ARS.host.*=.*/ARS.host = $addr/g" $ffile
    
}


# Update the Agent endpoint.properties file
updt_agent_endpt_prop () {
	local new_address=${1}
	
#/opt/ibm/director/agent/runtime/agent/config/endpoint.properties

    local props="PatchService.Host \
CatalogueService.Host \
CertManagement.Host \
AgentQuery.Host \
Registration.Server.Host \
TrustedCertificateQuery.Host \
AgentManagerQuery.Host"

    local src_dir=$INSTALL_DIR/agent/runtime/agent/config
    local filename=endpoint.properties
    local ffile=$src_dir/$filename
    
    debug "updt_agent_endpt_prop"
        
    if [ -f $ffile ]; then    	
        # Build the sed commands for all these props
	    build_agnt_sed_script 2 $new_address $props
    	
	    # Record the generated sed script
	    local sed_scr=$SED_SCRIPT
	                	        
        updt_prop_file $src_dir $filename "$sed_scr"
        
    	
    else
        debug "AgentManager file not found: $ffile"
        local file_name_parm=$ffile
        # TRANSLATORS: This error message means the file specified in the $file_name_parm was 
        # not found on the local file system
        sm_msg "`eval_gettext \"File not found: \\\$file_name_parm\"`"
    fi
   
}


get_agntmgr_addr () {

    # Make sure we have the Agent Mgr address.
    if [ -z "$agent_manager_address" ]; then
        # Try to get it from the config file        
        get_value_from_cfg agent-manager-address
        if [ -z "$CFG_RETURN_VALUE" ]; then
            debug "Could not determine Agent Manager Address"
            return 1
        else
            agent_manager_address=$CFG_RETURN_VALUE                    
        fi
        
    fi

	return 0
}

updt_agent_files () {
	local mode=${1}

    local endpoint_dir=$INSTALL_DIR/agent/runtime/agent/bin    
	
	# Before updating the file, stop the agent
    $endpoint_dir/endpoint.sh stop &> /dev/null

    updt_agent_endpt_prop $agent_manager_address
	put_am_addr_in_coreagent_dat $mode
	
	# Start the agent back up
    $endpoint_dir/endpoint.sh start &> /dev/null
	
}

updt_all_agntmgr_files_primary () {
	
    # Make sure we have the agent's address value
    get_agntmgr_addr
    
    if [ $? -eq 0 ]; then 
	    # Make the updates
	    updt_am_mgr_prop 
	    updt_am_agentmgr_prop
	    updt_agent_files primary
    fi
}

updt_all_agntmgr_files_secondary () {
	
    # Make sure we have the agent's address value
    get_agntmgr_addr
        
    if [ $? -eq 0 ]; then 
	    # Make the updates
	    updt_am_mgr_prop 
	    updt_am_agentmgr_prop
	    updt_agent_files secondary
    fi
    
}


################################################################

unmanage_agent () {
	/opt/ibm/director/agent/runtime/agent/toolkit/bin/configure.sh -force -unmanaged -nostart            
	
}


reset_mgmt_servers () {
	
    local srchd=/opt/ibm/director/agent/runtime/agent/subagents/
    local datfile=coreagent.dat

    debug "reset mgmt server...update coreagent.dat"
	cfile=`find $srchd -name "$datfile"`
	savfile=${cfile}.sav
	
	if [ -n "$cfile" ]; then
	    
	    rm -f ${savfile} 
	    mv ${cfile} ${savfile}
	    echo "ManagementServers=" >${cfile}
	    grep -v ManagementServers= ${savfile} >>${cfile}

        local result=`cat $cfile`
        debug "$result"        
	    
	else
	    fatal_error "coreagent.dat file not found"
	fi
	
}


get_host_address () {
    local name=${1}
    local hostNameFound=true
        
    HOST_ADDRESS=

    local max_tries=3
    local count=1
    while [ $count -le $max_tries ]
    do
        debug "get host address for $name"
        host_out=`host $name`
        local host_rc=$?
        debug "host_rc: $host_rc"
        host_addr=`echo $host_out | cut -d' ' -f4`
        debug "get_host_address rc: $host_rc host_addr is:$host_addr"
        if [ $host_rc -eq 0 ]; then
            debug "Address found: $host_addr"
            HOST_ADDRESS=$host_addr
            break
        elif [ "$host_addr" == "found:" ]; then
            debug "Name not found in dns"
            break        
        fi
        
        let "count = count + 1"
    done
    

    if [ -z "$HOST_ADDRESS" ]; then
        # The host command does not look at /etc/hosts file
        # PING pfeecl3lp5.in.ibm.com (9.126.134.214) 56(84) bytes of data.

        local pcmd=
        debug "host command didn't work, now try ping"
        ping -n -c 1 -W 1 -q $name 1>/dev/null 2>&1
        if [ $? != 2 ]; then
            pcmd=`ping -n -c 1 -W 1 -q $name 2>/dev/null | grep PING`
        else
            debug "ping command didn't work, now try ping6"
            ping6 -n -c 1 -W 1 -q $name 1>/dev/null 2>&1
            if [ $? != 2 ]; then
                pcmd=`ping6 -n -c 1 -W 1 -q $name 2>/dev/null | grep PING`
            else
                debug "ping6 command didn't work, can not get address for host $name"
                hostNameFound=false
            fi
        fi
        
        if [ "$hostNameFound" == "true" ]; then
            debug "output from ping command was: $pcmd"
            local h1=${pcmd#*(}
            local h2=${h1%%)*}
            debug "h2:$h2"
            HOST_ADDRESS=$h2
        fi
       
    fi
    
    debug "get_host_address: $name $HOST_ADDRESS"
     
    if [[ "$hostNameFound" != "true" || -z "$HOST_ADDRESS" ]]; then
        fatal_error "`eval_gettext \"Could not resolve address for host name \\\$name.  Make sure that the host name is in the DNS and that this system can connect to the DNS, or that the host name is in the hosts file.\"`"
    fi
    
}


put_am_addr_in_coreagent_dat () {
    local mode=${1}
    
    local srchd=/opt/ibm/director/agent/runtime/agent/subagents/
    local datfile=coreagent.dat

    debug "put agent mgmt server in coreagent.dat; $mode"
    cfile=`find $srchd -name "$datfile"`
    savfile=${cfile}.sav
    
    if [ -n "$cfile" ]; then
        
        rm -f ${savfile} 
        cp ${cfile} ${savfile}

        # substitute the agent manager float address for the host's static         
        local host_addr=
        if [[ "$mode" == "primary" ]]; then
        	get_host_address $HOST       	
            host_addr=$HOST_ADDRESS
        else
            get_node_from_daemon
            get_host_address $DAEMON_NODE
            host_addr=$HOST_ADDRESS
        fi
        
        debug "put agent mgmt server in coreagent.dat: host_addr: $host_addr am_addr:$agent_manager_address"
        local result=`cat $cfile 2>&1`
        debug "coreagent.dat contents before substitution:\n$result"        
        
        perl -pi -e "s/${host_addr//\./\\.}/$agent_manager_address/g" $cfile
        result=`cat $cfile 2>&1`
        debug "coreagent.dat contents after substitution:\n$result"
        cp ${cfile} ${cfile}.new        
        
    else
        fatal_error "coreagent.dat file not found"
    fi
    
}


get_node_from_daemon () {
	DAEMON_NODE=`grep node= $DM_CONF | cut -d= -f2`
}

register_agent_with_am () {
	local am_password=${1}

    get_node_from_daemon
    local am_host=$DAEMON_NODE	
	/opt/ibm/director/agent/runtime/agent/toolkit/bin/configure.sh -force -amhost $am_host -passwd $am_password
}


get_mep_id_for_host () {
	local name=${1}
    local addr=${2}
    MEP_ID=

#	cli_out=`smcli lssys -v -t OperatingSystem -o -i $addr`
#	mep_out=`echo $cli_out | grep $name | grep 0x | cut -d' ' -f2`

    get_cfg_user_id
    local cfg_user=$CFG_USER_ID

    local mep_out=`LC_ALL=C su - $cfg_user -c "/opt/ibm/director/bin/smcli lssys -v -t OperatingSystem -o -i $addr" | tail -1 | grep 0x | cut -d' ' -f2`
	
    debug "Get mep for $name, addr $addr is: $mep_out"
    if [ -n "$mep_out" ]; then
        let "MEP_ID=$mep_out"
        debug "mep_out:$mep_out MEP_ID:$MEP_ID"
	else
	    mep_out=`LC_ALL=C su - $cfg_user -c "/opt/ibm/director/bin/smcli lssys -v -t OperatingSystem -o -i $addr"`
	    debug "MEP ID not found for host $name address $addr:  mep_out:$mep_out"
    fi	
}

register_meps () {
	debug "Registering MEPS for $HOST and $node"
    register_mep $HOST
    register_mep $node		
}

register_mep () {
    local name=${1}
    
    get_host_address $name
    local addr=$HOST_ADDRESS
       
    wait_for_mep $name $addr 1200

# Pin HACK === TODO:  REMOVE after Agent gets fixed
#    pin_ipv6_address $name $MEP_ID     

	get_cfg_user_id
    local cfg_user=$CFG_USER_ID
    
    debug "register mep: ${name}=${addr}"
    su - $cfg_user -c "/opt/ibm/director/bin/smcli setManagementIP $MEP_ID  $addr"
}

wait_for_mep () {
    local name=${1}
    local addr=${2}
    local timeout_in_seconds=${3}
    local wait_forever=0
    local number_of_tries=1

    if [ -z "$timeout_in_seconds" ]; then
        timeout_in_seconds=-1
    fi

    if [ $timeout_in_seconds -eq 0 ]; then
        wait_forever=1
        debug "wait for mep, $name $addr about to loop until we find the meps"
    elif [ $timeout_in_seconds -gt 0 ]; then
        number_of_tries=`expr $timeout_in_seconds / 10`
        number_of_tries=`expr $number_of_tries + 1`
        debug "wait for mep, $name $addr about to loop for ${timeout_in_seconds} seconds"
    fi

    MEP_ID=
    for (( i = 0; i < ${number_of_tries}; i++ ))
    do
        get_mep_id_for_host $name $addr
        if [[ -n "$MEP_ID" ]]; then
            get_cfg_user_id
            local cfg_user=$CFG_USER_ID
            LC_ALL=C su - $cfg_user -c "/opt/ibm/director/bin/smcli lsresource OperatingSystem ${MEP_ID}" | grep AccessState | grep Unlocked
            if [ $? -ne 0 ]; then
                debug "OperatingSystem MEP $MEP_ID not unlocked yet"
            else
                debug "OperatingSystem MEP $MEP_ID is unlocked"
                return 0
            fi
        fi
        if [ $wait_forever == 1 ]; then
            debug "wait for mep, $name $addr ...retrying every 10 seconds until found and unlocked"
            i=-1
        else
            tryNumber=`expr $i + 1`
            debug "wait for mep, $name $addr ...retrying every 10 seconds for up to ${timeout_in_seconds} seconds, on try $tryNumber of $number_of_tries"
        fi
        sleep 10
        
        #allow developer to break out of this loop if necessary by setting a flag file
        if [ -f /opt/ibm/director/ha/config/.endWaitForMep ]; then
            i=${number_of_tries}
        fi
        
    done

    return 99
}


pin_ipv6_address () {
    local name=${1}
    local mep_id=${2}
    
    debug "HACK: pin ipv6 address"
    local tmpf=`mktemp /tmp/smha_pia.XXXXXX`  || fatal_error "`eval_gettext "Could not make temporary file"`"
    
   	get_cfg_user_id
    local cfg_user=$CFG_USER_ID

    LC_ALL=C su - $cfg_user -c "/opt/ibm/director/bin/smcli lsmeps" > $tmpf
    
    local foundMEP=0
    local foundGUID=0
    local GUID=
    while read -r line
    do
#    	debug "line:$line"
        case "$line" in
            "MEP: $name"*)
                debug "MEP line for $name:$line"
                foundMEP=1
            ;;
            GUID:*)
                if [ $foundMEP -eq 1 ]; then
                    debug "GUID line for $name:$line"
                    GUID=`echo $line | awk '{print $2}'`
                    debug "GUID is $GUID"
                    foundGUI=1
                    foundMEP=0
                fi
            ;;
            MEP*)
                foundMEP=0
            ;;
        esac
    done < $tmpf    
    rm $tmpf
    
    local addr=`LC_ALL=C su - $cfg_user -c "/opt/ibm/director/bin/smcli lssystemdb $GUID" | grep -e "IPv6Address" | grep value | grep -v {  | awk '{print $3}'`
    debug "address: $addr"
    
    debug "pinning $name: $MEP_ID $addr"
    su - $cfg_user -c "/opt/ibm/director/bin/smcli setManagementIP $MEP_ID  $addr"
    
}

turn_off_address_change_processing () {
	local prop=com.ibm.usmi.kernel.localsystem.USMSLocalSystemService.enableIPChangeProcessing
	
    if [ -f $USMIKERNEL_PROPS ]; then
    	local prop_present=`grep ${prop} $USMIKERNEL_PROPS`
    	if [ -n "$prop_present" ]; then
    	    perl -pi -e "s/^${prop}.*=.*/${prop}=false/g" $USMIKERNEL_PROPS
        else
            echo "${prop}=false" >> $USMIKERNEL_PROPS
        fi
    	 
    fi 
    debug "turn_off_address_change_processing called"
}

turn_on_address_change_processing () {
    local prop=com.ibm.usmi.kernel.localsystem.USMSLocalSystemService.enableIPChangeProcessing
    
    if [ -f $USMIKERNEL_PROPS ]; then
    	local prop_present=`grep ${prop} $USMIKERNEL_PROPS`
    	if [ -n "$prop_present" ]; then
    	    perl -pi -e "s/^${prop}.*=.*/${prop}=true/g" $USMIKERNEL_PROPS
        else
            echo "${prop}=true" >> $USMIKERNEL_PROPS
        fi
    fi 
    debug "turn_on_address_change_processing called"
}

force_address_change_processing_next_restart () {
    local prop1=com.ibm.usmi.kernel.localsystem.USMSLocalSystemService.forceIPChangeProcessing
    local prop2=com.ibm.isdvmi.ha.clearForceIPChangeAfterStartup
    
    if [ -f $USMIKERNEL_PROPS ]; then
    	local prop_present=`grep ${prop1} $USMIKERNEL_PROPS`
    	if [ -n "$prop_present" ]; then
    	    perl -pi -e "s/^${prop1}.*=.*/${prop1}=true/g" $USMIKERNEL_PROPS
        else
            echo "${prop1}=true" >> $USMIKERNEL_PROPS
        fi
        
        prop_present=`grep ${prop2} $USMIKERNEL_PROPS`
        if [ -n "$prop_present" ]; then
    	    perl -pi -e "s/^${prop2}.*=.*/${prop2}=true/g" $USMIKERNEL_PROPS
        else
            echo "${prop2}=true" >> $USMIKERNEL_PROPS
        fi
        
        
    fi 
    debug "force_address_change_processing_next_restart called"
}

remove_force_address_change_processing_property () {
    local prop1=com.ibm.usmi.kernel.localsystem.USMSLocalSystemService.forceIPChangeProcessing
    local prop2=com.ibm.isdvmi.ha.clearForceIPChangeAfterStartup
       
    if [ -f $USMIKERNEL_PROPS ]; then
    	local prop_present=`grep ${prop1} $USMIKERNEL_PROPS`
    	if [ -n "$prop_present" ]; then
    	    perl -pi -e "s/^${prop1}.*=.*/${prop1}=false/g" $USMIKERNEL_PROPS
        else
            echo "${prop1}=false" >> $USMIKERNEL_PROPS
        fi
        
        prop_present=`grep ${prop2} $USMIKERNEL_PROPS`
    	if [ -n "$prop_present" ]; then
    	    perl -pi -e "s/^${prop2}.*=.*/${prop2}=false/g" $USMIKERNEL_PROPS
        else
            echo "${prop2}=false" >> $USMIKERNEL_PROPS
        fi
    fi 
    debug "remove_force_address_change_processing_property called"
}

refresh_managementIP () {
    # We may need to update the preferred address for a node in 
    # director now.  If the "old address" was set as the management IP address
    # then it needs to be updated in order to access the CAS agent.  On a 
    # non-HA system, the "change IP address code" that Ashish Bilore's team did
    # would fix this but that code is disabled in an HA configuration, so we have
    # to handle it.  Note that the CAS endpoint needs to have been restarted for this to work
    # also (i.e. they needed to do the shutdown after the chnetcfg to change the address)
    # We need to check which node is the active node and do the processing there, 
    #since this requires smcli access.
    local target_host=$1
    local desired_address=$2
    debug "refresh_managementIP called with: $1: $2"
    is_passive
    if [ $? -eq 0 ]; then
    	# This is the active node
    	# If we just try to do the setManagementIP it will likely fail because director won't have a CAS RSAP for that
 		# IP address yet.  So, kick off a discovery first, and then check if we need to do a set.
 		get_cfg_user_id
    	local cfg_user=$CFG_USER_ID
    	debug "refresh_managementIP: cfg_user is $cfg_user" 
    	
 		local discover_out=`LC_ALL=C su - $cfg_user -c "/opt/ibm/director/bin/smcli discover -i $desired_address"`
 		debug "refresh_managementIP: discover output for $desired_address is: $discover_out"
 		
    	get_mep_id_for_host $target_host $desired_address
    	
    	debug "refresh_managementIP:  host: $target_host , desired address: $desired_address: , MEP_ID $MEP_ID"
    	# See if the desired address was set as the preferred address
    	if [ -n "$MEP_ID" ]; then 
    		# we found a MEP id for the desired address, so check it.
    		
    		local get_manIP_out=`LC_ALL=C su - $cfg_user -c "/opt/ibm/director/bin/smcli getManagementIP $MEP_ID"`
    		debug "refresh_managementIP: get_manIP_out is: $get_manIP_out"
    		# the output looks like: 
    		# OS MEP was found with OID: 2920
            # Management IP Address is: 9.5.36.77
            # except it is all merged into a single line when you echo it back.  
            # We need to pick up everything past the 2nd ":"...
            local temp=`echo ${get_manIP_out#*:}`
            #debug "temp: ->>$temp<<-"
            local current_preferred_ip=`echo ${temp#*:}`
 			debug "refresh_managementIP: current_preferred_ip: -->>$current_preferred_ip<<--"
    		if [ -n "$current_preferred_ip" ]; then
 				if [ "$current_preferred_ip" == "$desired_address" ]; then
 					debug "refresh_managementIP: no need to update preferred management IP address for host $target_host"
 				else
 					
 					debug "refresh_managementIP: $desired_address differs from set address $current_preferred_ip: attempting to setManagementIP to $desired_address for MEP $MEP_ID"
 					# Update it to the new address.
 					local set_ip_out=`LC_ALL=C su - $cfg_user -c "/opt/ibm/director/bin/smcli setManagementIP $MEP_ID $desired_address"`
 					debug "refresh_managementIP: setManagementIP output is: $set_ip_out"
 				fi
 			else
 				debug "refresh_managementIP: could not determine preferred management IP for MEP $MEP_ID"
 			fi
 		else
 			debug "refresh_managementIP: could not determine the MEP for desired address $desired_address" 
 		fi
 	else
 		debug "refresh_managementIP: running on passive node - will not attempt to issue setManagementIP on this node."
 	fi
 	
}

refresh_managementIPs () {
	debug "refresh_managementIPs called"
	is_passive
    if [ $? -eq 0 ]; then
    	# This is the active node
		# call once for each node.  We want the preferred management IP address for the MEP for each node to be set
		# to whatever the output of "hostname -i" would be on that node.
		# First, call for the local node.	
		local desired_address=`hostname -i`
		refresh_managementIP $HOST $desired_address
	
		# Now, figure out the info for the other node.
		# Get other node name
		get_value_from_cfg node
		local other_node_hostname=${CFG_RETURN_VALUE}
		# get IP address other node's hostname resolves to.  It is set in HOST_ADDRESS
		get_host_address_etc_hosts_override $other_node_hostname
		refresh_managementIP $other_node_hostname $HOST_ADDRESS
	else
		debug "passive node - send a update local agents request to other side"
		run_init "updatelocalagents" 
		
	fi
}

set_cas_ip_property () {
	debug "set_cas_ip_property called"

    local cas_ip=${1}
    local endpoint_dir=$INSTALL_DIR/agent/runtime/agent/bin    
	
    if [ "${2}" == "restartAgent" ]; then
        # Before updating the file, stop the agent
    	debug "Stopping CAS agent"
        stop_output=`$endpoint_dir/endpoint.sh stop 2>&1`
    	debug "CAS agent stopped"
    	debug "$stop_output"
    fi
    
    if [ -f $TWGNEW_PROP_FILE ]; then
        debug "Found twgnew.properties file: contents include:"
        twgnewFileContents=`cat $TWGNEW_PROP_FILE`
        debug "$twgnewFileContents"
    else
        debug "twgnew.properties file not found, so creating it now"
        touch $TWGNEW_PROP_FILE
        chown root:root $TWGNEW_PROP_FILE
        chmod 600 $TWGNEW_PROP_FILE
    fi
    
    local old_cas_ip=`grep cas.ip $TWGNEW_PROP_FILE | cut -d= -f2`
    if [ -n "${old_cas_ip}" ]; then
        perl -pi -e "s/^cas.ip.*=.*/cas.ip=${cas_ip}/g" $TWGNEW_PROP_FILE
        # set old_cas_ip as preha.ip.cas (so it is there on unconfigure to revert back to)
        echo "preha.ip.cas=${old_cas_ip}" >> $TWGNEW_PROP_FILE
    else
        echo "cas.ip=${cas_ip}" >> $TWGNEW_PROP_FILE
        echo "preha.ip.cas=NONE" >> $TWGNEW_PROP_FILE
    fi

    if [ "${2}" == "restartAgent" ]; then
    	# Start the agent back up
    	debug "Starting CAS agent"
    	start_output=`$endpoint_dir/endpoint.sh start 2>&1`
    	debug "CAS agent started:"
    	debug "$start_output"
    fi
}

revert_cas_ip_property () {
    debug "revert_cas_ip_property called"

    local endpoint_dir=$INSTALL_DIR/agent/runtime/agent/bin    
    local old_cas_ip=

    if [ "${1}" == "restartAgent" ]; then
        # Before updating the file, stop the agent
        debug "Stopping CAS agent"
        stop_output=`$endpoint_dir/endpoint.sh stop 2>&1`
        debug "CAS agent stopped"
        debug "$stop_output"
    fi
    
    if [ ! -f $TWGNEW_PROP_FILE ]; then
        debug "twgnew.properties file not found, so nothing to revert to"
    else
        debug "Found twgnew.properties file: contents include:"
        twgnewFileContents=`cat $TWGNEW_PROP_FILE`
        debug "$twgnewFileContents"

        old_cas_ip=`grep preha.ip.cas $TWGNEW_PROP_FILE | cut -d= -f2`
        if [ -n "${old_cas_ip}" ]; then
            if [ "${old_cas_ip}" == "NONE" ]; then
                # remove cas.ip property that was set on HA Config
                sed -i "/cas.ip/d" $TWGNEW_PROP_FILE
            else
                # revert cas.ip property to the value in preha.ip.cas
                perl -pi -e "s/^cas.ip.*=.*/cas.ip=${old_cas_ip}/g" $TWGNEW_PROP_FILE
            fi
                
            # remove preha.ip.cas property
            sed -i "/preha.ip.cas/d" $TWGNEW_PROP_FILE
        else
            debug "preha.ip.cas not found in $TWGNEW_PROP_FILE, so no need to revert the cas.ip property.  Most likely it was set manually by customer or service"
        fi
    fi
    
    if [ "${1}" == "restartAgent" ]; then
        # Start the agent back up
        debug "Starting CAS agent"
        start_output=`$endpoint_dir/endpoint.sh start 2>&1`
        debug "CAS agent started:"
        debug "$start_output"
    fi
}
