# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996
#

DHCP_INIT_CMD=/etc/init.d/dhcpd
DHCP_BIN_CMD=/usr/sbin/dhcpd
DHCP_ETC_CONF=/etc/dhcpd.conf
DHCP_ETC_SYSCONFIG_CONF=/etc/sysconfig/dhcpd

DHCP_MIRROR=/mirror0/os/dhcp
DHCP_NON_MIRROR=/etc/nonmirror/os/dhcp
DHCP_MIRROR0_CONF=/mirror0/os/dhcp/etc/dhcpd.conf
DHCP_MIRROR0_SYSCONFIG_CONF=/mirror0/os/dhcp/etc/sysconfig/dhcpd


kill_dhcp () {
	local dhcp_pid=`ps auxww | grep -i dhcpd.leases | grep chroot | grep sbin/dhcpd | awk '{print $2}'`
	if [ -n "$dhcp_pid" ]; then
		debug "killing dhcp pid:$dhcp_pid"
		kill $dhcp_pid
	fi
}



####
# This function will set the correct paths for DHCP
#
# DHCP_NONSHARED - equals either 1 or 0
# DHCP_BASE - points to the base of either the mirror or non-mirror path
# DHCP_PID_FILE - points to proper pid file
# DHCP_CONF - points to the proper conf file
#
###
common_dhcp_path_setup  () {
    #DHCP_PID_FILE=/mirror0/os/dhcp/var/lib/dhcp/var/run/dhcpd.pid
    #DHCP_CONF=/etc/dhcpd.conf

    debug "DHCP_NONSHARED=$DHCP_NONSHARED"
    # Get the shared value if not already defined
    if [ -z "$DHCP_NONSHARED" ]; then
        get_value_from_cfg dhcp
        debug "CFG_RETURN_VALUE=$CFG_RETURN_VALUE"    
        case $CFG_RETURN_VALUE in
        "shared" )
            debug "DHCP server config is shared"
            DHCP_NONSHARED=0
        ;;
        "nonshared" )
            debug "DHCP server config is non shared"
            DHCP_NONSHARED=1
        ;;
        esac
        
    fi
    
    debug "DHCP_NONSHARED=$DHCP_NONSHARED"
    if [ $DHCP_NONSHARED -eq 1 ]; then
        DHCP_BASE=$DHCP_NON_MIRROR                        
    else
        DHCP_BASE=$DHCP_MIRROR
    fi


    if [ ! -d $DHCP_BASE ]; then
    	debug "Making directory: $DHCP_BASE"
        mkdir -p $DHCP_BASE
    fi

    # Setup the variables    
    DHCP_PID_FILE=$DHCP_BASE/var/lib/dhcp/var/run/dhcpd.pid
    DHCP_CONF=$DHCP_BASE/etc/dhcpd.conf
    DHCP_SYSCONFIG_CONF=$DHCP_BASE/etc/sysconfig/dhcpd
    
    #debug "DHCP_PID_FILE=$DHCP_BASE/var/lib/dhcp/var/run/dhcpd.pid"
    #debug "DHCP_CONF=$DHCP_BASE/etc/dhcpd.conf"
    #debug "DHCP_SYSCONFIG_CONF=$DHCP_BASE/etc/sysconfig/dhcpd"

    debug "exiting dhcp path setup:`ls -lah $DHCP_ETC_CONF`"
    debug "exiting dhcp path setup:`ls -lah $DHCP_ETC_SYSCONFIG_CONF`"
    
}

##
# Check the /etc/dhcp.conf file and make sure it's
# a link to the correct file
##
check_dhcp_conf_file () {
    debug "Check DHCP conf file"   
    
    # If the /etc/dhcpd.conf doesn't exit
    if [ ! -e $DHCP_ETC_CONF ]; then
    	# create the link
    	debug "The dhcp.conf file didn't exist. Create link for DHCP file"
        ln -sf $DHCP_CONF $DHCP_ETC_CONF        	
    fi
    
    if [ ! -L $DHCP_ETC_CONF ]; then
        debug "DHCP config is not a link"
#        if [ ! -e $DHCP_CONF ]; then
        if [ $DHCP_CONF -ot $DHCP_ETC_CONF ]; then
            debug "DHCP Mirror file does not exist or is older, move it."
            mkdir -p $(dirname $DHCP_CONF)
            local rm_out=`LC_ALL=C rm -f $DHCP_CONF 2>&1`
            debug "rm -f $DHCP_CONF : $rm_out"
            local mv_out=`LC_ALL=C mv $DHCP_ETC_CONF $DHCP_CONF 2>&1`
            debug "mv $DHCP_ETC_CONF $DHCP_CONF : $mv_out"
        fi
        
        debug "Create link for DHCP file"
        ln -sf $DHCP_CONF $DHCP_ETC_CONF    
        
    fi
    
    # Now make sure the link points to the right spot.
    # Do this by listing the link and searching for the base path
    local containslink=`ls -l $DHCP_ETC_CONF | grep $DHCP_BASE`
    if [ -z "$containslink" ]; then
        # The link doesn't contain the base path
        debug "DHCP file link was pointing to the wrong config base, remove it and create the link"
        rm -f $DHCP_ETC_CONF
        ln -sf $DHCP_CONF $DHCP_ETC_CONF            
    fi        
    debug "Exiting check_dhcp:`ls -lah $DHCP_ETC_CONF`"
}


##
# Check the /etc/sysconfig/dhcpd file and make sure it's
# a link to the correct file
##
check_sysconfig_dhcp_file () {
    debug "Check sysconfig dhcp conf file"   
    
    # If the /etc/sysconfig/dhcpd doesn't exit
    if [ ! -e $DHCP_ETC_SYSCONFIG_CONF ]; then
        # create the link
        debug "The sysconfig/dhcp file didn't exist. Create link"
        ln -sf $DHCP_SYSCONFIG_CONF $DHCP_ETC_SYSCONFIG_CONF            
    fi
    
    if [ ! -L $DHCP_ETC_SYSCONFIG_CONF ]; then
        debug "sysconfig/dhcp  is not a link"
#        if [ ! -e $DHCP_SYSCONFIG_CONF ]; then
        if [ $DHCP_SYSCONFIG_CONF -ot $DHCP_ETC_SYSCONFIG_CONF ]; then
            debug "sysconfig/dhcp Mirror file does not exist or is older, move it."
            mkdir -p $(dirname $DHCP_SYSCONFIG_CONF)
            local rm_out=`LC_ALL=C rm -f $DHCP_SYSCONFIG_CONF 2>&1`
            debug "rm -f $DHCP_SYSCONFIG_CONF : $rm_out"
            local mv_out=`LC_ALL=C mv $DHCP_ETC_SYSCONFIG_CONF $DHCP_SYSCONFIG_CONF 2>&1`
            debug "mv $DHCP_ETC_SYSCONFIG_CONF $DHCP_SYSCONFIG_CONF : $mv_out" 
        fi
        
        debug "Create link for sysconfig/dhcp file"
        ln -sf $DHCP_SYSCONFIG_CONF $DHCP_ETC_SYSCONFIG_CONF    
        
    fi
    
    # Now make sure the link points to the right spot.
    # Do this by listing the link and searching for the base path
    local containslink=`ls -l $DHCP_SYSCONFIG_CONF | grep $DHCP_BASE`
    if [ -z "$containslink" ]; then
        # The link doesn't contain the base path
        debug "dhcp sysconf file link was pointing to the wrong config base, remove it and create the link"
        debug "ln -sf $DHCP_SYSCONFIG_CONF $DHCP_ETC_SYSCONFIG_CONF"
        rm -f $DHCP_ETC_SYSCONFIG_CONF
        ln -sf $DHCP_SYSCONFIG_CONF $DHCP_ETC_SYSCONFIG_CONF            
    fi
            
    debug "Exiting check_sysconfig_dhcp:`ls -lah $DHCP_ETC_SYSCONFIG_CONF`"
    
}


#/mirror0/os/dhcp/var/lib/dhcp on /var/lib/dhcp type none (rw,bind)
#/mirror0/os/dhcp/var/lib/dhcpv6 on /var/lib/dhcpv6 type none (rw,bind)
#lrwxrwxrwx 1 root root   32 2010-06-09 22:39 /etc/dhcp6s.conf -> /mirror0/os/dhcp/etc/dhcp6s.conf

mv_dhcp_from_mirror_to_nonmirror () {
	debug "Moving dhcp data from mirrored to non mirrored storage"
	
	if [ -d ${DHCP_NON_MIRROR} ]; then
	   rm -rf ${DHCP_NON_MIRROR}/*
    fi
	mv $DHCP_MIRROR/* $DHCP_NON_MIRROR
}

mv_dhcp_from_nonmirror_to_mirror () {
    debug "Moving dhcp data from non mirrored to mirrored storage"
    
    if [ -d ${DHCP_MIRROR} ]; then
        rm -rf ${DHCP_MIRROR}/*
    fi
    mv $DHCP_NON_MIRROR/* $DHCP_MIRROR
}

chg_dhcp_bndmounts_mirror_to_nonmirror () {
    debug "Changing bind mounts from mirrored to non mirrored storage"

    # older three disk file layout
    perl -pi -e "s#^/mirror0/os/dhcp#/etc/nonmirror/os/dhcp#g" /opt/ibm/vmi/data/disklayout.cfg
    # newer 7 disk layout format
    perl -pi -e "s#^bind:/mirror0/os/dhcp#bind:/etc/nonmirror/os/dhcp#g" /opt/ibm/vmi/data/disklayout.cfg    
    /opt/ibm/vmi/bin/updatefstab -w        
}

chg_dhcp_bndmounts_nonmirror_to_mirror () {
	debug "Changing bind mounts from non mirrored to mirrored storage"	

    # older three disk file layout
    perl -pi -e "s#^/etc/nonmirror/os/dhcp#/mirror0/os/dhcp#g" /opt/ibm/vmi/data/disklayout.cfg
    # newer 7 disk layout format
    perl -pi -e "s#^bind:/etc/nonmirror/os/dhcp#bind:/mirror0/os/dhcp#g" /opt/ibm/vmi/data/disklayout.cfg
    /opt/ibm/vmi/bin/updatefstab -w
}

check_dhcp_bndmounts () {
	# Our changes to go from mirrored to nonmirrored might get wiped out. Check them.
    if [ $DHCP_NONSHARED -eq 1 ]; then
    	debug "Check bind mounts for nonshared dhcp"
    	local nonmirrored=`grep "^/etc/nonmirror"  /etc/fstab | grep dhcp | grep bind`
    	if [ -z "$nonmirrored" ]; then
    		chg_dhcp_bndmounts_mirror_to_nonmirror
    	fi
    fi
}


remount_dhcp_bind_mounts () {
	debug "remount dhcp bind mounts"
	
	#debug "stop dhcpd"
	# $SMHA_SCRIPTS/tsa/smhatsadhcp.sh stop
	$SMHA_DIR/plugins/drbd/smhasyslog.sh stop "/mirror0"
	kill_dhcp
	
    local dhcp_mounts=`mount | grep dhcp | grep bind | cut -d' ' -f1`
    for mnt in $dhcp_mounts
    do
    	debug "Unmounting $mnt"
        local cmd_out=`LC_ALL=C umount $mnt 2>&1`
        if [ -n "$cmd_out" ]; then
            debug "umount result:$cmd_out"
        fi
    	
    done

    local dhcp_bindmounts=`grep dhcp /etc/fstab | grep bind | awk '{print $2}'`
    for bndmnt in $dhcp_bindmounts
    do
        debug "Mounting $bndmnt"
        local cmd_out=`LC_ALL=C mount $bndmnt 2>&1`
        if [ -n "$cmd_out" ]; then
            debug "mount result:$cmd_out"
        fi
        
    done

    $SMHA_DIR/plugins/drbd/smhasyslog.sh start "/mirror0"
    debug "done remounting dhcp bind mount"
    	
}

is_dhcp_configd () {
    #local result=`chkconfig dhcpd | grep " on"`
    #local result=`lsnetcfg -n | grep dhcpserver | grep on`
    local result=`LC_ALL=C lsnetcfg -n -F dhcpstatus | grep "on"`
    
    debug "is_dhcp_configd: result:$result"
    if [ -L /etc/dhcpd.conf ]; then
    	debug "/etc/dhcpd.conf link found."
    else
        debug "/etc/dhcpd.conf not link found."
    fi
    
#    local result=`LC_ALL=C lsnetcfg -n -F dhcpstatus | grep "on"`    
    if [ -n "$result" -a -e /etc/dhcpd.conf ]; then
    	debug "is_dhcp_configd=1"    
       return 1
    else
       return 0
    fi
}

crt_dhcp_rsrc_if_on () {
    
    is_dhcp_configd
    if [ $? -eq 1 ]; then    
        crt_dhcp_rsrc_step1 $dhcp
    fi
}

# Pass either "shared" or "nonshared"
crt_dhcp_rsrc_step1 () {
	debug "Creating dhcp resource step 1."
    local sharing=${1}

    $DHCP_INIT_CMD stop
    
    case $sharing in
    "shared" )
        DHCP_NONSHARED=0
    ;;
    "nonshared" )
        DHCP_NONSHARED=1
    ;;
    *)
        debug "crt_dhcp_rsrc_step1 called with unknown sharing value: $sharing"
        # default to shared configuration
        DHCP_NONSHARED=0
    ;;
    esac

    if [ -z "$node" ]; then
        get_value_from_cfg node
        if [ -n "$CFG_RETURN_VALUE" ]; then
            # We now have the node name
            node=$CFG_RETURN_VALUE
        else
            fatal_error "Could not retrieve the node value from configuration."
        fi
    fi
    
    # Setup all our dhcp variables
    common_dhcp_path_setup
    
    # move the data from shared to non-shared if needed.
    # before dealing with the conf file setup
    if [ $DHCP_NONSHARED -eq 1 ]; then
        mv_dhcp_from_mirror_to_nonmirror        
    fi
        
    # Make sure the /etc/dhcp.conf file is setup correctly.
    check_dhcp_conf_file
    
    # Make sure the /etc/sysconfig/dhcp file is setup correctly.
    check_sysconfig_dhcp_file

    # move the data from shared to non-shared if needed.
    if [ $DHCP_NONSHARED -eq 1 ]; then    	
    	chg_dhcp_bndmounts_mirror_to_nonmirror

        remount_dhcp_bind_mounts
        
        dhcp_rsrc_type=2
        
        # Call the secondary node to setup it's dhcp config
        run_init "dhcp nonshared" 
                            	
    fi
    
    crt_dhcp_rsrc $sharing
    
}

crt_dhcp_rsrc () {
    sharing=${1}
    debug "Creating dhcp resource. Sharing=$sharing"
    
    case $sharing in
    "shared" )
        DHCP_NONSHARED=0
        local dhcp_rsrc_type=1
    ;;
    "nonshared" )
        DHCP_NONSHARED=1
        local dhcp_rsrc_type=2
    ;;
    *)
        debug "crt_dhcp_rsrc called with unknown sharing value: $sharing"
        # default to shared configuration
        DHCP_NONSHARED=0
    ;;
    esac
	
	# Only create the resource if it doesn't already exist
    lsrg -A d -g $DHCP_RG &>/dev/null
    if [ $? -eq 6 ]; then
        debug "Create the dhcp resources type:$dhcp_rsrc_type"
        
        get_local_rsct_node_name
        local rsct_node=$LOCAL_RSCT_NODE_NAME

        get_remote_rsct_node_name
        local remote_rsct_node=$REMOTE_RSCT_NODE_NAME

        mkrsrc IBM.Application \
        Name="$DHCP_RSRC_NAME" \
        StartCommand="$DHCP_MONITOR_SCRIPT start" \
        StopCommand="$DHCP_MONITOR_SCRIPT stop" \
        MonitorCommand="$DHCP_MONITOR_SCRIPT status" \
        MonitorCommandPeriod=$DHCP_MON_CMD_PERIOD \
        MonitorCommandTimeout=$DHCP_MON_CMD_TIMEOUT \
        NodeNameList="{'$rsct_node','$remote_rsct_node'}" \
        StartCommandTimeout=$DHCP_MON_START_TIMEOUT \
        StopCommandTimeout=$DHCP_MON_STOP_TIMEOUT \
        UserName="root" \
        ResourceType=$dhcp_rsrc_type

#        CleanupNodeList="{'$rsct_node','$remote_rsct_node'}" \
#        CleanupCommand="$SMHA_SCRIPTS/$DHCP_MONITOR_SCRIPT cleanup" \
#        CleanupCommandTimeout=$CLEANUP_COMMAND_TIMEOUT \

        # create the resource group
        mkrg $DHCP_RG
        if [ $dhcp_rsrc_type -eq 2 ]; then
        	chrg_l_none $DHCP_RG       
        fi
        add_to_rg $DHCP_RSRC_NAME IBM.Application $DHCP_RG $ADDRGMBR_ORDER
        
        DHCP_RESOURCES_CREATED=1
        
    else
        debug "dhcp resources already created"   
    fi  
	
}

crt_dhcp_rsrc_step2 () {
	
	if [ -z "$DHCP_RESOURCES_CREATED" ]; then
		DHCP_RESOURCES_CREATED=0
	fi
	if [ $DHCP_RESOURCES_CREATED -eq 1 ]; then
	    # We need this in a separate routine since we need to 
	    # call it after DRBD is created.	
	    if [ $DHCP_NONSHARED -eq 0 ]; then        
	        # DHCP depends on the drbd storage being mounted                    
	        make_relationship DependsOn IBM.Application:${DHCP_RSRC_NAME} \
	        IBM.Application:${DHCPMIRROR_RSRC_NAME} \
	        ${DHCP_RSRC_NAME}-depon-${DHCPMIRROR_RSRC_NAME}
	    fi
        # Don't bring the rg online.  It will be done later.	    
    fi
    
}


dlt_dhcp_rsrc () {

    is_rg_created $DHCP_RG
    if [ $? -eq 1 ]; then
        take_rg_offline_n_wait $DHCP_RG
        
        rmrsrc -s "Name=='$DHCP_RSRC_NAME'" IBM.Application Force=1        
        
        rmrg $DHCP_RG
                        
    fi

}

terminate_dhcpd__restore_orig_config () {

    # We have no indication of whether this is a shared or non-shared config
    #   or even if dhcp was ever turned on, but we only need to restore the
    #   config if it was a non-shared config
    
    local containslink=`ls -l $DHCP_ETC_CONF | grep $DHCP_NON_MIRROR`
    if [ -n "$containslink" ]; then
    	# Move the data from the non-mirror back to the mirror and restore the link
    	mv_dhcp_from_nonmirror_to_mirror
    	
    	chg_dhcp_bndmounts_nonmirror_to_mirror
    	
        rm -f $DHCP_ETC_CONF
        ln -sf $DHCP_MIRROR0_CONF $DHCP_ETC_CONF            
            
        rm -f $DHCP_ETC_SYSCONFIG_CONF
        ln -sf $DHCP_MIRROR0_SYSCONFIG_CONF $DHCP_ETC_SYSCONFIG_CONF
    	    	
    fi
    	
}


crt_dhcp6_rsrc () {
    # Only create the resource if it doesn't already exist
    lsrg -A d -g $DHCP6_RG &>/dev/null
    if [ $? -eq 6 ]; then
        debug "Create the dhcp6 resources"
        
        get_local_rsct_node_name
        local rsct_node=$LOCAL_RSCT_NODE_NAME

        get_remote_rsct_node_name
        local remote_rsct_node=$REMOTE_RSCT_NODE_NAME
        
        mkrsrc IBM.Application \
        Name="$DHCP6_RSRC_NAME" \
        StartCommand="$DHCP_MONITOR_SCRIPT start 6" \
        StopCommand="$DHCP_MONITOR_SCRIPT stop 6" \
        MonitorCommand="$DHCP_MONITOR_SCRIPT status 6" \
        MonitorCommandPeriod=$DHCP_MON_CMD_PERIOD \
        MonitorCommandTimeout=$DHCP_MON_CMD_TIMEOUT \
        NodeNameList="{'$rsct_node','$remote_rsct_node'}" \
        StartCommandTimeout=$DHCP_MON_START_TIMEOUT \
        StopCommandTimeout=$DHCP_MON_STOP_TIMEOUT \
        UserName="root" \
        ResourceType=1
        
        # create the resource group
        mkrg $DHCP6_RG
        add_to_rg $DHCP6_RSRC_NAME IBM.Application $DHCP6_RG $ADDRGMBR_ORDER
        sleep 5
        # chrg -o online $DHCP6_RG
        debug "tsa call hook 9"
        local tsa_result
        tsa_result=$(call_tsa_chrg_o_l_u -o online $DHCP6_RG)
        local RC=$?
        if [ $RC -ne 0 ]; then
            debug "RC=$RC  $tsa_result"
        fi
    else
        debug "dhcp6 resources already created"   
    fi  
}

dhcp_secondary_nonshared_init () {
	
	debug "setup for dhcp nonshared on secondary"
	
	DHCP_NONSHARED=1
	
	# Setup all our dhcp variables
    common_dhcp_path_setup
    
    mv_dhcp_from_mirror_to_nonmirror
        
    # Make sure the /etc/dhcp.conf file is setup correctly.
    check_dhcp_conf_file
    
    # Make sure the /etc/sysconfig/dhcp file is setup correctly.
    check_sysconfig_dhcp_file        
    
    chg_dhcp_bndmounts_mirror_to_nonmirror

    remount_dhcp_bind_mounts
}


record_dhcp_autostart () {
	local flag=`LC_ALL=C /sbin/chkconfig dhcpd | awk '{print $2}' | grep -i on`
	if [ -n "$flag" ]; then
		touch $SMHA_CONFIGDIR/.dhcpstart
	fi
}

restore_dhcp_if_needed () {
	if [ -e $SMHA_CONFIGDIR/.dhcpstart ]; then
		debug "Restoring DHCP"
		/sbin/chkconfig --level 35 dhcpd on &>/dev/null
		/sbin/service dhcpd start
		rm -f $SMHA_CONFIGDIR/.dhcpstart
	fi
}

remove_dhcp_autostart () {

    debug "Remove DHCP autostart"
    /sbin/chkconfig -d dhcpd &>/dev/null
}

#restore_dhcp_runlevel () {
#    
#    lsrg -A d -g $DHCP_RG &>/dev/null
#    if [ $? -ne 6 ]; then
#        debug "Restoring DHCP run levels"
#        /sbin/chkconfig --level 35 dhcpd on &>/dev/null
#    fi
#}

