#!/bin/bash
# Licensed Materials - Property of IBM
#
# 09N6996
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

PATH=/bin:/usr/bin:/usr/sbin/rsct/bin
PROGNAME=$(basename $0)
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh
INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv          # required by smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi

load_common_code $INSTALL_DIR/ha/bin/include/smhaplugin.sh

#############################################################

# if current time is less than previous time, return an error

compare_TS () {
    local rc=0

    if (( CurTSsec < PrevTSsec )); then
        rc=1
    elif (( CurTSsec == PrevTSsec )); then
        if (( CurTSmic < PrevTSmic )); then
            rc=1
        fi
    fi

    return $rc
}

OPSTATE_FILE=$SMHA_CONFIGDIR/drbdmOpState

NodeName=$1
TimeOut=$2

if [ $ERRM_TYPEID -eq 2 ]; then
    debug "Error Event: $ERRM_TIME $ERRM_ERROR_NUM $ERRM_ERROR_MSG"

else
    debug "$ERRM_TIME $ERRM_RSRC_NAME $ERRM_NODE_NAMELIST $ERRM_VALUE"

    # get a write lock to serialize among multiple instances of this script

    if rw_lock write get drbdmEvent $PROGNAME 60
    then
        WriteEvent=1
        if [ -s $OPSTATE_FILE ]; then

            # Unfortunately, for events from the same resource, the EventResponse scripts are invoked
            # as the event arrives, so if a response script is slow in returning, this script could be
            # invoked in parallel for a subsequent event. Therefore, compare the current and previous
            # event time stamps. If the previous event is "newer" discard the current event.

            read PrevTimeStamp junk < $OPSTATE_FILE
            PrevTSsec=${PrevTimeStamp%,*}
            PrevTSmic=${PrevTimeStamp#*,}

            CurTSsec=${ERRM_TIME%,*}
            CurTSmic=${ERRM_TIME#*,}

            if [ -z "$PrevTSsec" -o -z "$PrevTSmic" -o -z "$CurTSsec" -o -z "$CurTSmic" ]; then
                fatal_error "Invalid timestamps: ${PrevTSsec}/${PrevTSmic}/${CurTSsec}/${CurTSmic}"
            fi
            if ! compare_TS ; then

                # it is possible time has moved backward, so check the cached event against
                # the current system time

                REAL_TIME=$(date '+%s,%N')
                CurTSsec=${REAL_TIME%,*}
                CurTSmic=${REAL_TIME#*,}
                CurTSmic=${CurTSmic:0:6}     # truncate nanoseconds to microseconds

                if compare_TS ; then
                    WriteEvent=0
                fi
            fi

        fi

        if [ $WriteEvent -eq 1 ]; then
            echo  "$ERRM_TIME $ERRM_RSRC_NAME $ERRM_VALUE" > $OPSTATE_FILE

            if [ $ERRM_VALUE -eq $TSA_ONLINE ]; then
                # the local mirror0 is online; run the plug-ins

                run_plugins event "${SMHA_EVENT_MIRRORED_STORAGE} up"

            elif [ $ERRM_VALUE -eq $TSA_OFFLINE ]; then
                # the local mirror0 is down; run the plug-ins

                run_plugins event "${SMHA_EVENT_MIRRORED_STORAGE} down"
            fi
        else
            debug "Discard stale event"
        fi
        rw_lock write free drbdmEvent $PROGNAME
    else
        fatal_error "Cannot obtain lock !!!"
    fi
fi

exit 0
