#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996
#
PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################


PARM_TRANSLATION="n name"
PARM_VALID=$PARM_TRANSLATION


load_common_code $INSTALL_DIR/ha/bin/include/smhaparms.sh


delete_user() {
	debug "issuing delete for user ${name}"
	/opt/ibm/director/vmi/scripts/execWithPrivilege/rmsmusr --name ${name}
	RC=$?
	debug "delete for user ${name} completed with rc=$RC"
	return $RC
}

kill_soft() {
   debug "attempting to kill any login processes for user $name"
   who -u | grep ${name} | awk '{print $7}' | xargs kill
}


#################################################
#  Main code
#################################################

if [ $# -eq 0 ]; then
    exit 0
fi

# smhadltusr.sh --name=somename
parse_all_parms $*
RC=0

if [ -n "$name" ]; then
	for i in {1..5}
	do
		delete_user
		RC=$?
		# Try to delete any logged in users with this name.  We'll try this 
		# an arbitrary number of times.  
		# Note - I tested using ps -aef and killing with -9 if the account still cannot
		# be deleted after issuing a soft kill, , but then info for the login shell stays in /var/run/utmp and the
		# account cannot be deleted, even though no running processes are left.  So, not coding any additional recovery if this
		# fails.
		if [ $RC -eq 0 ]; then
			break;
		fi 
		if [ $RC -eq 11 ]; then
			sleep 1
			kill_soft
		fi
	done
fi

exit $RC



