#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996
#

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

#TODO: change these to a relative path

INSTALL_DIR=/opt/ibm/director
STATUSFILE=$INSTALL_DIR/data/usmsstatus.txt
UPDATESTATUSFILE=$INSTALL_DIR/data/updates-installing.tmp

export TEXTDOMAINDIR=/usr/share/locale
export TEXTDOMAIN=smhacmd
. /usr/bin/gettext.sh

PORT=2035

# Return codes
RC_ACTIVE=0
RC_STARTING=1
RC_ENDING=2
RC_INACTIVE=3
RC_ERROR=4
RC_HELP=5
RC_UPDATING=6
RC_BADPARMS=7

REPEAT=0
PREVRC=-9999


readStatusFiles() {
   if [ -f $UPDATESTATUSFILE ]
   then
     return $RC_UPDATING
   fi

   RC=$RC_INACTIVE
   if [ -f $STATUSFILE ]
   then
     local contents=`cat $STATUSFILE`

     if [ "$contents" == "Starting" ]
     then
       RC=$RC_ERROR
     elif [ "$contents" == "Active" ]
     then
       RC=$RC_ERROR
     elif [ "$contents" == "Inactive" ]
     then
       RC=$RC_INACTIVE
     elif [ "$contents" == "Stopping" ]
     then
       RC=$RC_ERROR
     fi

   fi

   return $RC
}

directorStatus() {
   local sock_rc
   # Save stderr to 4 and redirect stderr to null
   exec 4>&2 2<>/dev/null
   exec 3<>/dev/tcp/127.0.0.1/$PORT
   sock_rc=$?
   # Restore stderr and close 4
   exec 2>&4 4>&-
   
   
   # if the tcp connection failed, attempt to read the status files
   if [ $sock_rc -ne "0" ]
   then
     readStatusFiles
     return $?
   fi
   
   echo -e "list">&3

   local match="0" 
   local status=
   local response=
   local read_errors=0 

   while [ $match -eq "0" ]
   do
     read -e status <&3 2> /dev/null
     read_rc=$?
     if [ $read_rc -eq 0 ]; then
       response=$response$status
       match=`expr match "$status" ".*over"`
     else
       let "read_errors += 1"
       if [ $read_errors -gt 5 ]; then
         break
       fi
     fi
   done

   echo -e "exit">&3  
   match=`expr match "$response" ".*Transport Active"` 
   
   if [ $match -gt "0" ]
   then
     match=`expr match "$response" ".*Server Active"`

     if [ $match -gt "0" ]
     then
       return $RC_ACTIVE
     fi
    
     match=`expr match "$response" ".*Server Stopping"` 
     if [ $match -gt "0" ]
     then
       return $RC_ENDING
     fi
     return $RC_STARTING
   fi

   match=`expr match "$response" ".*Transport Stopping"` 
   if [ $match -gt "0" ]
   then
     return $RC_ENDING
   fi

   match=`expr match "$response" ".*Transport Starting"`
   if [ $match -gt 0 ]
   then
     return $RC_STARTING
   fi

   match=`expr match "$response" ".*Server Starting"`
   if [ $match -gt 0 ]
   then
     return $RC_STARTING
   fi
   return $RC_INACTIVE 
}

#TODO: nls enable
displayStatusMsg() {
  curRC=$1
  if [ "$PREVRC" -ne "$curRC" ]
  then
    case $curRC in
      $RC_ACTIVE)
        # TRANSLATORS: This indicates that the management server is Active on this node
        echo "`eval_gettext "Active"`"
        ;;
      $RC_STARTING)
        # TRANSLATORS: This indicates that the management server is Starting on this node
        echo "`eval_gettext "Starting"`"
        ;;
      $RC_ENDING)
        # TRANSLATORS: This indicates that the management server is Ending on this node
        echo "`eval_gettext "Ending"`"
        ;;
      $RC_INACTIVE)
        # TRANSLATORS: This indicates that the management server is Inactive on this node
        echo "`eval_gettext "Inactive"`"
        ;;
      $RC_ERROR)
        # TRANSLATORS: This indicates an error in HA configuration or data mirroring
        echo "`eval_gettext "Error"`"
        ;;
      $RC_UPDATING)
        # TRANSLATORS: This indicates that the management server is Updating 
        # itself with new code on this node
        echo "`eval_gettext "Updating"`"
        ;;
    esac
  fi
  PREVRC=$curRC
}

main() {
  directorStatus
  local RC=$?
  displayStatusMsg $RC
  while [ $REPEAT -gt 0 ]
  do
    sleep 5s
    directorStatus
    RC=$?
    displayStatusMsg $RC
  done
  return $RC
}

usage () {
	echo "$PROGNAME [-h][-r]"
}

while getopts "hr" OPTION
do
  case $OPTION in
    h)
      usage
      exit
      ;;
    r)
      REPEAT=1
      ;;
    ?)
      usage
      exit
      ;;
  esac
done

main
exit $?


