#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996
#

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)
#### Use a special path for TSA monitoring scripts
PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/sbin/rsct/bin
export PATH

#### Standard code to find an pull in all sm common code ####
# Use the specified path to this command, then working directory, then path
PATH=$PROGPATH:`pwd`:$PATH
CMDFINDER=which
CMD_DIR=`$CMDFINDER $PROGNAME`
CMD_DIR=`dirname $CMD_DIR`

COMMON_CODE=$CMD_DIR/../include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################


RC=$TSA_UNKNOWN  

action=${1}
srv=${2}

# Construct cmd
SRV_CMD=/etc/init.d/${srv}

# Check for supported services
case ${srv} in
    cimserverd) ;;
    cimlistenerd) ;;
    ibmsa)
        if [ ! -e /etc/init.d/ibmsa ]; then
        	srv=paslpd
        fi
    ;;
    paslpd) ;;
    tier1slpinst) ;;
    *) debug "Unsupported service:${srv}"; exit ${TSA_UNKNOWN} ;;
esac

# Handle the action
case ${action} in
start)
    
    $SRV_CMD start
    sm_sys_log "$srv started"
    RC=0
    ;;
stop)
    $SRV_CMD stop
    
    sm_sys_log "$srv stopped"
    RC=0
    ;;

status)

    $SRV_CMD status 2>&1 >/dev/null
    statusrc=$?
    case $statusrc in
        0) #Active
            RC=${TSA_ONLINE}
            ;;
        1) #Active for tier1slpinst
            case "$srv" in            
                tier1slpinst)  RC=${TSA_ONLINE} ;;
#                paslpd)  RC=${TSA_ONLINE} ;;
                *) RC=${TSA_OFFLINE} ;;
            esac
            ;;
            
        3) #Inactive
            RC=${TSA_OFFLINE}
            ;;
        *) # ??? Inactive
            RC=${TSA_OFFLINE}
            ;;
    esac
    ;;
esac
record_tsa_action $action cim-$srv $RC
exit $RC
