# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996
#

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)
#### Use a special path for TSA monitoring scripts
PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/sbin/rsct/bin
export PATH

#### Standard code to find an pull in all sm common code ####
INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/tsa
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################
load_common_code $INSTALL_DIR/ha/bin/include/smhadhcp.sh


#################################################
#  Functions
#################################################



#################################################
#  Main code
#################################################

RC=$TSA_UNKNOWN
  
Action=${1}

case ${Action} in
start)
    debug "DHCP server start"
    
    # Setup all our dhcp variables
    common_dhcp_path_setup
    
    # Make sure the setup is correct.
    check_dhcp_conf_file
    check_sysconfig_dhcp_file
    
    # Make sure the bind mounts are in the fstab file
    check_dhcp_bndmounts
    
    # Make sure the mounts are setup
    remount_dhcp_bind_mounts
    
    debug "Starting dhcp server"
    dhcp_out=`$DHCP_INIT_CMD start 2>&1`
    debug "DHCP server start output: $dhcp_out"
    sm_sys_log "DHCP started"
    RC=0
    ;;
stop)
    debug "DHCP server stop"
    $DHCP_INIT_CMD stop
    
    sm_sys_log "DHCP stopped"
    RC=0
    ;;

status)

    if [ ! -e $SYSLOG_STOPPED_DHCP ]; then
        /etc/init.d/dhcpd status
        statusrc=$?
        case $statusrc in
            0) #Active
                RC=${TSA_ONLINE}
                ;;
            *) #Inactive
                RC=${TSA_OFFLINE}
                ;;
        esac
    else
        # Syslog stopped dhcp to unmount so stalllllll
        # Unknown will cause TSA to stop automating it temporarily
        RC=${TSA_UNKNOWN}
    fi    
    ;;
cleanup)
    debug "DHCP failed."
    ;;    
esac
record_tsa_action $Action dhcp $RC
exit $RC
