#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996
#

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################

mk_daemon () {
    for i in {1..100}
    do
        debug "Create the HA daemon resources"
        mkrsrc IBM.Application \
        Name="$HADM_RSRC_NAME" \
        StartCommand="$SMHA_SCRIPTS/$HADM_MONITOR_SCRIPT start" \
        StopCommand="$SMHA_SCRIPTS/$HADM_MONITOR_SCRIPT stop" \
        MonitorCommand="$SMHA_SCRIPTS/$HADM_MONITOR_SCRIPT status" \
        MonitorCommandPeriod=$HADM_MON_CMD_PERIOD \
        MonitorCommandTimeout=$HADM_MON_CMD_TIMEOUT \
        NodeNameList="{'$HOST','$node'}" \
        StartCommandTimeout=$HADM_MON_START_TIMEOUT \
        StopCommandTimeout=$HADM_MON_STOP_TIMEOUT \
        UserName="root" \
        ResourceType=2
        
        rsrc_exists $HADM_RSRC_NAME IBM.Application
        if [ $? -eq 1 ]; then
            return 0
        else
            debug "Retry making rsrc $HADM_RSRC_NAME"
            sleep 5       
        fi
    done
}

mk_mq () {
    for i in {1..100}
    do
        debug "Create the HA MQ resources"
    
        mkrsrc IBM.Application \
        Name="$HAMQ_RSRC_NAME" \
        StartCommand="$SMHA_SCRIPTS/$HAMQ_MONITOR_SCRIPT start" \
        StopCommand="$SMHA_SCRIPTS/$HAMQ_MONITOR_SCRIPT stop" \
        MonitorCommand="$SMHA_SCRIPTS/$HAMQ_MONITOR_SCRIPT status" \
        MonitorCommandPeriod=$HAMQ_MON_CMD_PERIOD \
        MonitorCommandTimeout=$HAMQ_MON_CMD_TIMEOUT \
        NodeNameList="{'$HOST','$node'}" \
        StartCommandTimeout=$HAMQ_MON_START_TIMEOUT \
        StopCommandTimeout=$HAMQ_MON_STOP_TIMEOUT \
        UserName="root" \
        ResourceType=2

        rsrc_exists $HAMQ_RSRC_NAME IBM.Application
        if [ $? -eq 1 ]; then
            return 0
        else
            debug "Retry making rsrc $HAMQ_RSRC_NAME"
            sleep 5       
        fi
    done
}


crt_daemon_rsrc () {
    # Only create the resource if it doesn't already exist
    lsrg -A d -g $HADM_RG &>/dev/null
    if [ $? -eq 6 ]; then
    	
    	mk_daemon
    	mk_mq
    	
        debug "daemon resources created"
        # create the resource group
        mkrg -l none $HADM_RG
        add_to_rg $HADM_RSRC_NAME IBM.Application $HADM_RG
        
        # The daemon is running so we need to take it offline before we can add the mq resource
        chrg -o offline $HADM_RG
        sleep 2
        add_to_rg $HAMQ_RSRC_NAME IBM.Application $HADM_RG
       
#        sleep 5
#        chrg -o online $HADM_RG
        start_n_wait_for_rg_online $HADM_RG
        
    else
        debug "HA daemon resources already created"   
    fi  
}


#### Main ########


method=$1

case ${method} in
create)
    crt_daemon_rsrc
    ;;        
delete)
    debug "Delete for daemon called."
    # We don't delete anything, just let it get deleted when the domain is deleted.
    ;;        
*)
    fatal_error "Bad parameter to $PROGNAME"
    ;;        
esac

