#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)
EXTPROGNAME="smha"

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################

PARM_FLAGS="h help s start p stop S suspend R resume r reset F failover v validate c cancel T tsa M meps i immediate A forceActive X cleanup"
PARM_MULTI=""
PARM_TRANSLATION="h help \
s start \
p stop \
S suspend \
R resume \
r reset \
T tsa \
M meps \
F failover \
v validate \
c cancel \
i immediate \
A forceActive \
X cleanup \
C configure-rg \
E enable-rg \
D disable-rg \
" 
PARM_SINGLES="help start stop suspend resume cleanup configure-rg enable-rg disable-rg"
PARM_VALID=$PARM_TRANSLATION
PARM_DONT_SAVE="help start stop suspend resume reset failover validate cancel immediate forceActive cleanup configure-rg enable-rg disable-rg"

load_common_code $INSTALL_DIR/ha/bin/include/smhaparms.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaerrcode.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaagent.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadrbd.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhainitcom.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaplugin.sh

###################################
## Functions                     ##
###################################
proc_parms () {
	
	# Parse the parameters
	# The result should be that variables are set for each parameter and
	# CMD_PARMS will contain the list of parameters processed
	# If a file is specified it will also read the file.
    parse_all_parms $*
    
	
    for option in $CMD_PARMS
    do
    	# For each option, make sure the minimum parameters are specified
	    case $option in 
	        validate)
	           	ensure_parms_xor "failover" "forceActive"
	        ;;
	        "cancel")
	           	ensure_parms "failover"
	        ;;
	    esac
    done

    
}

echo_help () {
    # TRANSLATORS: This is linux command syntax, do not translate these words since they are parameter names
    echo -e "`eval_gettext \"\\\${EXTPROGNAME}: [-h|--help][-s|--start][-p|--stop][-S|--suspend][-R|--resume][-r|--reset][-F|--failover [ [-v|--validate] | [-c|--cancel] ] ][-A|--forceActive [-v|--validate] ]\"`"
    # TRANSLATORS: This is linux command syntax, only translate Name. Name is the parameter value that specifies a resource group name
    echo -e "`eval_gettext "      [-C Name|--configure-rg=Name][-E Name|--enable-rg=Name][-D Name|--disable-rg=Name]"`"
    echo -e "`eval_gettext \"The \\\${EXTPROGNAME} command is used to manage high availability.\"`"
    # TRANSLATORS: "Options:" is a header that indicates that below are command options that can be use for this command
    echo -e "`eval_gettext "Options:\n"`"
    # TRANSLATORS: This is a parameter definition for a linux command
    echo -e "`eval_gettext "\t-h,--help\t\tDisplay this help"`"
    # TRANSLATORS: This is a parameter definition for a linux command, this option lets users start the high 
    # availability environment if it has been stoped
    echo -e "`eval_gettext "\t-s,--start\t\tStart high availability"`"
    # TRANSLATORS: This is a parameter definition for a linux command, this option lets users stop the high 
    # availability environment
    echo -e "`eval_gettext "\t-p,--stop\t\tStop high availability"`"
    # TRANSLATORS: This is a parameter definition for a linux command, this option lets users stop the 
    # management server but not stop high availability completely
    echo -e "`eval_gettext "\t-S,--suspend\t\tSuspend the management server"`"
    # TRANSLATORS: This is a parameter definition for a linux command, this option lets the user start
    # the management server if it has been suspended (stopped) by the --suspend option)
    echo -e "`eval_gettext "\t-R,--resume\t\tResume the management server"`"
    # TRANSLATORS: This is a parameter definition for a linux command, this option lets the user reset
    # the definition of all the HA resources that might be in a failed state
    echo -e "`eval_gettext "\t-r,--reset\t\tReset high availability resources"`"
    # TRANSLATORS: This is a parameter definition for a linux command
    echo -e "`eval_gettext "\t-F,--failover\t\tFailover to the passive node (unless -c or -v is specified)"`"
    # TRANSLATORS: This is a parameter definition for a linux command
    echo -e "`eval_gettext "\t-c,--cancel\t\tAttempt to cancel a failover that is in progress"`"
    # TRANSLATORS: This is a parameter definition for a linux command
    echo -e "`eval_gettext "\t-A,--forceActive\tAttempt to force this node to become the active node (unless -v is specified).  Used when --failover will not work."`"
    # TRANSLATORS: This is a parameter definition for a linux command
    echo -e "`eval_gettext "\t-v,--validate\t\tCheck for conditions that do not allow a successful --failover or --forceActive from working"`"
    # TRANSLATORS: This is a parameter definition for a linux command
    echo -e "`eval_gettext "\t-X,--cleanup\t\tCleanup an unconfigured node after a failed high availability configuration attempt.  Do not run this while the high availability configuration is still running or on a node that has already been configured for high availability."`"
    # TRANSLATORS: This is a parameter definition for a linux command where Name is the name of a resource group
    echo -e "`eval_gettext "\t-C,--configure-rg\tConfigure for high availability the subsystem specified by the resource group Name."`"
    # TRANSLATORS: This is a parameter definition for a linux command where Name is the name of a resource group
    echo -e "`eval_gettext "\t-E,--enable-rg\t\tEnable the subsystem specified by the resource group Name. The resource group is set online."`"
    # TRANSLATORS: This is a parameter definition for a linux command where Name is the name of a resource group
    echo -e "`eval_gettext "\t-D,--disable-rg\t\tDisable the subsystem specified by the resource group Name. The resource group is set offline.\n"`"
}

#####################################################
# Process the help parameter
#####################################################
proc_help () {
	
	if [[ "$help" == "Y" ]]; then
		echo_help
        exit 0
	fi

}

chk_ha_setup () {
	is_domain_setup
    if [ $? -eq 0 ]; then
        fatal_error_rc $HA_ENVIRONMENT_NOT_CONFIGURED "`eval_gettext "The high availability environment has not been configured."`"
    fi
	
}

AUTOMATION_STATE=       # set from results of is_automation_on: 0=off, 1=on
check_automation () {

    is_automation_on
    AUTOMATION_STATE=$?
    if [ $AUTOMATION_STATE -gt 1 ]; then
        # Bad return code
        debug "is_automation_on returned: $rc"
        fatal_error_rc $UNKNOWN_ERROR "`eval_gettext "An unknown error occurred.  Retry the operation, and if the problem persists reboot both nodes and then try the operation again."`"
    fi
}

#####################################################
# Process the stop parameter
#####################################################
proc_stop () {
    
    if [ -n "$stop" ]; then
        # The stop paramter was specified
        
        chk_ha_setup
                
        # the state of "stopped" in the configuration of the resource groups, i.e. NominalState is Offline.
        
        take_all_rg_offline
        # Give them some time before we start polling if they are down
        sleep 15
        wait_all_rg_offline
        
        local lrc=$?
        if [ $lrc -eq 0 ]; then
            # TRANSLATORS: This message indicates that the smha --stop command worked successfully
        	sm_msg "`eval_gettext "High availability has stopped."`"
        else
            sm_log "Return code from wait_all_rg_offline: $lrc"
        	fatal_error "`eval_gettext "An error occurred while stopping high availability.  Retry the operation, and if the problem persists, reboot both nodes and then try the operation again."`"
    	fi
    	
    	log_command_success        
        exit 0
    fi
    
}

#####################################################
# Process the start parameter
#####################################################
proc_start () {
    
    if [ -n "$start" ]; then
        # The start paramter was specified
        
        chk_ha_setup
                
        check_PD_status
        local lrc=$?
        if [ $lrc -ne 0 ]
        then
            sm_log "Return code from check_PD_status: $lrc"
        	fatal_error "`eval_gettext "An error occurred while starting high availability.  Retry the operation, and if the problem persists, reboot both nodes and then try the operation again."`"
        fi

        # Start all rg
        start_all_rg_online                   
                                
        # TRANSLATORS: This message indicates that the smha --start command completed successfully
        sm_msg "`eval_gettext "High availability has started."`"
        log_command_success
        exit 0
    fi
    
}


#####################################################
# Process the suspend parameter
#####################################################
proc_suspend () {
    local RC
    local RGlist
    local RGname
    
    if [ -n "$suspend" ]; then
    	
        chk_ha_setup
                
        # find all resource groups that contains resources that depend on the director resource

        RGlist="$(find_Dependent_RGs $DIRECTOR_RSRC_NAME)"
        RC=$?

        if [ $RC -eq 1 ]; then
            # TRANSLATORS: This message indicates that the smha --suspend command could not
            # successfully determine what resources it needed to suspend
            fatal_error "`eval_gettext "Could not determine dependent resources."`"
        fi

        if [ $RC -eq 0 ]; then
            # take these resource groups offline so the director resource will go offline
            for RGname in ${RGlist}
            do
                take_rg_offline $RGname
            done
        fi

        # We can't keep the state of suspend in the config.
        # That's because they could be doing it from either node and the cfg's would be out of sync
        
        take_rg_offline $DIRECTOR_RG
        sleep 5
        wait_for_rg_offline $DIRECTOR_RG
        
        # (OEMB)
        # TRANSLATORS: This message indicates that the smha --suspend command completed successfully
        sm_msg "`eval_gettext "The management server is suspended."`"
        log_command_success
        exit 0
    fi
    
}

#####################################################
# Process the resume parameter
#####################################################
proc_resume () {
    
    if [ -n "$resume" ]; then
        chk_ha_setup
                       
        # Start all rg
        start_all_rg_online                   
                                
        # (OEMB)
        # TRANSLATORS: This message indicates that the smha --suspend command completed successfully
        sm_msg "`eval_gettext "The management server has resumed."`"
        log_command_success
        exit 0
    fi
    
}

####################################################

in_move () {
	
    local in_move=`LC_ALL=C lssam | grep "$STORAGE_RG" | grep "Move"`
    if [ -n "$in_move" ]; then
        return 1               	
    fi
    
    return 0
	
}

all_mirrors_uptodate () {

    # Assume all are uptodate
    local rc=1
    
    # Get the list of mirrors        
    local tmpf=`mktemp /tmp/smha_amu.XXXXXX`  || fatal_error "`eval_gettext "Could not make temporary file"`"
    $GETMIRROR_CMD > $tmpf
    while read -r line
    do
        local mdev=`echo $line | cut -d' ' -f1` 
        local mmir=`echo $line | cut -d' ' -f2`
        local mlab=${mmir:1}

        is_drbd_mirror_uptodate $mlab
        if [ $? -eq 0 ]; then
        	debug "Not all mirrors are up to date"
        	rm -f $tmpf  >/dev/null        	
        	return 0
        fi
                 
    done < $tmpf
        
    rm -f $tmpf  >/dev/null
    return $rc

}

is_drbd_mirror_uptodate () {
	local mirror=$1
	local rc=0
    local mirror_state="UpToDate/UpToDate"     # set if we are emulating resources
	
    if [ -z "$SMHA_EMULATED_RSRCS" ]
    then
        mirror_state=`drbd-overview | grep $mirror | awk '{print $4}'`
    fi
    debug "Mirror state for $mirror is:$mirror_state"
    case "$mirror_state" in         
        UpToDate/UpToDate)            
            rc=1
            ;;
        *)
            rc=0
            ;;
    esac
    return $rc
	
}

confirm_all_in_sync () {
    local timeout_in_seconds=${1}
    local wait_forever=0
    local number_of_tries=1

    if [ -z "$timeout_in_seconds" ]; then
        timeout_in_seconds=-1
    fi

    if [ $timeout_in_seconds -eq 0 ]; then
        wait_forever=1
        debug "confirm_all_in_sync, $name $addr about to loop until we find the meps"
    elif [ $timeout_in_seconds -gt 0 ]; then
        number_of_tries=`expr $timeout_in_seconds / 10`
        number_of_tries=`expr $number_of_tries + 1`
        debug "confirm_all_in_sync, $name $addr about to loop for ${timeout_in_seconds} seconds"
    fi

	local i
    for (( i = 0; i < ${number_of_tries}; i++ ))
    do
        all_mirrors_uptodate
		if [ $? -ne 0 ]; then
			# Just return if they disks are up to date
			return
		fi
		
		# sleep for a while so that the disks can sync some more
		if [ `expr $i + 1` -lt $number_of_tries ]; then
		    debug "Manual failover: Disks not in sync.  Sleeping"
		    sleep 10
        fi
    done
    
    #if we get to this code, it means the data is still not in sync
    # TRANSLATORS: "not synchronized" here means that changed data on the active node has not been copied to the passive node yet.
    fatal_error_rc $FAILOVER_FAILED_DATA_NOT_SYNCD "`eval_gettext "Data is not synchronized.  Wait until the data is synchronized and then try the operation again.  Use smhastatus -m to monitor data synchronization."`"
}

process_move () {
    # See if it's already in a move
    in_move
    if [ $? -eq 1 ]; then
        fatal_error_rc $ALREADY_IN_FAILOVER_ATTEMPT "`eval_gettext "A fail over attempt is already in progress. Use -c to cancel."`"
    fi
    
    confirm_all_in_sync 100
    
    # Move the resource groups to the other node
    local move_out=`LC_ALL=C rgreq -o move $STORAGE_RG 2>&1`
    debug "failover -move cmd:$? output:\n $move_out"
}

#####################################################
# Process the failover parameter
#####################################################
proc_failover () {

    chk_ha_setup
    
    # Make sure we're in dual node mode
    is_single_node_mode
    if [ $? -eq 1 ]; then
    	set_dual_node_mode
    fi

    # See if automation is off and if it is, just turn it on.
    check_automation
    case $AUTOMATION_STATE in
        0 )
            turn_on_automation
        ;;
        1 ) # Automation is on, check if we're excluded.
            is_excluded_list_empty
            if [ $? -eq 0 ]; then
            	debug "Some nodes are excluded."
            	include_all_nodes
            else
                process_move
            fi
        ;;
    esac    

    log_command_success
    exit 0
		
}

#####################################################
# Process the cancel parameter
#####################################################
proc_cancel () {
    
    if [[ "$cancel" == "Y" ]]; then
    	
    	chk_ha_setup
    	
        # See if it's already in a move
        in_move
        if [ $? -eq 0 ]; then
            # TRANSLATORS: This message indicates that the smha --failover --cancel command can 
            # not complete successfully because there is no failover to cancel.
            fatal_error "`eval_gettext "A fail over is not in progress."`"
        fi
        # Cancel the Move
        local move_out=`LC_ALL=C rgreq -o movecancel $STORAGE_RG`
    	debug "failover -movecancel cmd:$? output:\n $move_out"
    	
    	log_command_success
        exit 0
    fi

}


#####################################################
# Process the validate parameter
#####################################################

# the following function exits on error

validate_forceActive () {
    local RC
    local OpState

    chk_ha_setup

    is_passive
    RC=$?
    if [ $RC -ne 1 ]; then
        fatal_error "`eval_gettext "The --forceActive parameter may only be specified on the passive node."`"
    fi

    check_automation
    if [ $AUTOMATION_STATE -eq 1 ]; then

        get_local_rsct_node_name
        local lsrg_out=$(LC_ALL=C lsrg -s 'Name LIKE "%"' 2> /dev/null | fgrep ExcludedList | fgrep $LOCAL_RSCT_NODE_NAME)
        debug "lsrg_out: $lsrg_out"
        if [ -z "$lsrg_out" ]; then
            fatal_error "`eval_gettext "Only specify the --forceActive parameter if the active node did not restart after an update or if the active node could not be restarted after running the smshutdown command with no failover."`"
        fi
    fi

    get_remote_rsct_node_name
    OpState=$(LC_ALL=C lsrsrc-api -s "IBM.PeerNode::Name == '$REMOTE_RSCT_NODE_NAME'::OpState" 2> /dev/null)
    if [ -z "$OpState" ] || [ $OpState -ne 2 -a $OpState -ne 6 ]; then
        fatal_error "`eval_gettext "The --forceActive parameter may only be specified if the active node has failed."`"
    fi
}

proc_validate () {
    
    if [[ "$validate" == "Y" ]]; then
    	
        if [ -n "$failover" ]; then

	        chk_ha_setup
    	
    	    # Check to see if there are any reasons we believe the fail over will not be successful
            # Perform any tests here
            chk_nodes_opstate

            if [[ "$immediate" == "Y" ]]; then
                confirm_all_in_sync 1
            else
                confirm_all_in_sync 100
            fi
        else    # must be forceActive, based on parm checking for validate
            validate_forceActive
        fi
        
        # If we make it here, everything must be okay
        # TRANSLATORS: This message indicates that the smha --validate command found no errors
        sm_msg "`eval_gettext "No validation errors found."`"
        
        log_command_success
        exit 0
    fi

}


# Reset any resources that need it
reset_rsrcs () {
	local name
	local opstate
	local nodelist
	local resource
	local reset_rcrs=N
	
	local tmpf=`mktemp /tmp/smha_rr.XXXXXX`  || fatal_error "`eval_gettext "Could not make temporary file"`"
	LC_ALL=C lsrsrc IBM.Application Name OpState NodeNameList ResourceType > $tmpf
	LC_ALL=C lsrsrc IBM.ServiceIP Name OpState NodeNameList ResourceType >> $tmpf
	while read -r line
	do
		case "$line" in
			Name*)
			     name=`echo $line | cut -d' ' -f 3`
			     #debug "name: $name"   
            ;;
			OpState*)
			     opstate=`echo $line | cut -d' ' -f3`
			     #debug "opstate: $opstate"
            ;;
			NodeNameList*)
			     nodelist=`echo $line | cut -d' ' -f3`
			     #debug "nodelist: $nodelist"			     
			;;
			ResourceType*)
		         resource=`echo $line | cut -d' ' -f3`
                 # Resource is the last attribute in the groupings so process the group:
                 # Only process fixed resources (float have fixed too!)
                 if [ "$resource" == "0" ]; then
                    debug "possible reset: $name $opstate $nodelist"
                 	case $opstate in
                 		# 0=unknown,1=online,2=offline,3=failed offline,4=stuck online,5=pending online,6=pending offline
                 		3|4)                 		     
                 		     # Get the node name from the node list form. Delete everything up to the first "
                             node=${nodelist#*\"}
                             node=${node%\"*}
                             node=`echo $node | cut -d'.' -f1`
                 		     is_node_online $node
                 		     if [ $? -eq 1 ]; then 
                   		         debug "Reseting ==> Node: $nodelist Opstate: $opstate : $name"
                 		         if [[ "$name" == \"alias* ]]; then
                 		         	 debug "Reseting IBM.ServiceIP name is: $name"
                         		     reset_out=`LC_ALL=C resetrsrc -s "Name=$name" IBM.ServiceIP 2>&1`
                         		     debug "Reset rc:$? out:$reset_out"
                      		     else
                      		         debug "Reseting IBM.Application $name on nodelist $nodelist"
                         		     reset_out=`LC_ALL=C resetrsrc -s "Name=$name && NodeNameList=$nodelist" IBM.Application 2>&1`
                         		     debug "Reset rc:$? out:$reset_out"
                   		         fi
                                 reset_rcrs=Y
                 		     fi       		     
                 		;;
#                 		*)
#                 		     debug "OpState is: $opstate"
#                 		;;
                 	esac
                 fi
			;;
		esac
	done < $tmpf	
	rm $tmpf
	
	# If we didn't find any resources that were failed, try taking director offline and back online, if it isn't online
	if [ "$reset_rcrs" == "N" ]; then

        # First, make sure the Director resource isn't already online (ie working)		
		is_director_online
	    if [ $? -eq 0 ]; then
	    	
			# Second, make sure that Director's RG isn't already offline, if that's the case then
			# we don't want to create the side effect of bring it online 
			get_rg_status_num $DIRECTOR_RG
			if [ $? -ne 2 ]; then 
			
			  take_rg_offline_n_wait $DIRECTOR_RG
			
			  ch_rg_online $DIRECTOR_RG
			fi
		fi
		
	fi
		
}

reset_tsa () {
	
    local pid=`ps aux | grep IBM.RecoveryR | grep rsct | awk '{print $2}'`

    if [ -n "$pid" ]; then
        kill $pid
    else
        debug "IBM.RecoveryRMd process not found."
    fi
}

reset_meps () {

    # need $HOST and $node defined.	
    if [ -z "$node" ]; then
        get_value_from_cfg node
        if [ -n "$CFG_RETURN_VALUE" ]; then
            # We now have the node name
            node=$CFG_RETURN_VALUE
        else
            # TRANSLATORS: This message indicates that the smha command could not retrieve the passive node name from
            # a confguration file
            fatal_error "`eval_gettext "Could not retrieve the node value from configuration."`"
        fi
    fi
    
    $SMHA_DIR/bin/tsa/smhadirstatus.sh 1>/dev/null 2>&1
    if [ $? -eq 0 ]; then
	    register_meps
    else
        debug "Cannot reregister meps...director is not active."
    fi
	
	
}

#####################################################
# Process the reset parameter
#####################################################
proc_reset () {
    
    if [[ "$reset" == "Y" ]]; then
    	
    	chk_ha_setup

        # delete any tsa blocking files that we find    	    	
        rm -f /var/ct/samtb_net_blockreserve_*    	    	
    	    	
        if [[ "$tsa" == "Y" ]]; then
        	reset_tsa
        elif [[ "$meps" == "Y" ]]; then
            reset_meps
        else
            reset_rsrcs        
        fi
    	
        # TRANSLATORS: This message indicates that the smha --reset command completed successfully
    	sm_msg "`eval_gettext "High availability resources have been reset."`"
    	log_command_success
        exit 0
    fi

}


#####################################################
# Process the forceActive parameter
#####################################################
proc_forceActive () {
    if [[ "$forceActive" == "Y" ]]; then
        validate_forceActive    # this function exits on error. AUTOMATION_STATE is set by this function
        if [ $AUTOMATION_STATE -eq 0 ]; then

            # ensure critical resource protection is enabled
            set_crit_rsrc_protection $RSCT_CRIT_RSRC_PROTECT__HARD_RESET

            turn_on_automation
        else

            if [ -e $DRBD_NOSTART_FLAG ]; then

                # The presence of the .drbd.NoStart flag indicates an upgrade was in process on the
                # active node (this must be the passive node, given the call to validate_forceActive).
                # The DRBD base resource should be failed offline. Reset the resources before attempting
                # to brin the stack up on this node.

                rm -f $DRBD_NOSTART_FLAG    # remove flag file first, so reset can start drbd base
                reset_rsrcs
            fi

            debug "Remove this node from RG ExcludedList to bring up the stack hereon."

            include_this_node
        fi

        log_command_success
        exit 0
    fi
}


#####################################################
# Process the cleanup parameter
#####################################################
proc_cleanup () {

    if [[ "$cleanup" == "Y" ]]; then
        debug "smha --cleanup called"
        is_HAconfigured
        if [ $? -eq 0 ]; then
            # try 4 times, 6 seconds apart, to find an ha related script running
            for i in 1 2 3 4
            do
                #Look for an smha script that would indicate that HA is being configured (not smhamqrun.sh, smhadmrun.sh, smhap.sh, smhachkspool.sh, or smhadirstatus.sh though)
                grepOutput=`ps -ef | grep -e "smha.*[.]sh" | grep -v grep | grep -v "run[.]sh" | grep -v "smhadirstatus[.]sh" | grep -v "smhachkspool[.]sh" | grep -v "smhap[.]sh"`
                if [ $? -eq 0 ]; then
                    sm_msg "`eval_gettext "smha --cleanup will not be run because a high availability process is running.  If you are sure that high availability configuration is not running, reboot this node and try the command again."`"
                    debug "Process Info: $grepOutput"
                    exit 1
                elif [ $i -lt 4 ]; then
                    debug "try $i to find smha process, nothing found, sleeping for 6 seconds before trying again"
                    sleep 6
                fi
            done

            debug "About to remove persistent data, .cfgstatus, smhadm.conf, broker.ks, .errorDuringConfig"
            # Remove any MQ data
            rm -rf $SMHA_DIR/persist/*
            # Remove files indicating previous HA config error (if they exist)
            rm -rf $ERROR_DURING_CONFIG_FLAGFILE
            rm -rf $CFG_STATUS_FILE
            rm -rf $DM_CONF
            rm -rf $BROKER_KS
        else
            # TRANSLATORS: This message indicates that the smha --cleanup command can't be run because 
            # high availability is already successfully configured
            sm_msg "`eval_gettext "High availability is configured, so cleanup will not be run."`"
            exit 2
        fi

        exit 0
    fi
}

#####################################################
# Process the configure-rg parameter
#####################################################
proc_configure_rg () {
    local OKcount=0
    local RGNames
    local RGName
    local Owner
    local RC

    if [ -n "$configure_rg" ]; then
        debug "smha --configure-rg $configure_rg called"

    	chk_ha_setup

        RGNames=${configure_rg//,/ }     # replace commas by spaces
        set $RGNames

        while [ -n "$1" ]; do
            RGName=$1
            debug "Configure subystem specified with resource group name $RGName"

            # invoke configuration script for the RG on the remote node
            if run_init configInit controlled "${RGName}" ; then

                if run_controlled_plugins configinit $RGName ; then

                    if run_controlled_plugins create $RGName ; then
                        sm_msg "`eval_gettext \"Successfully configured the resource group \\\"\\\${RGName}\\\".\"`"
                        (( OKcount++ ))

                        # If this RG is Enabled, start it

                        Owner=$(LC_ALL=C lsrsrc-api -s "IBM.ResourceGroup::Name == \"${RGName}\"::Owner" 2> /dev/null)
                        RC=$?
                        debug "RG=$RGName  Owner=$Owner"
                        if [ $RC -eq 0 -a "$Owner" = "Enabled" ]; then
                            ch_rg_online $RGName
                        fi
                    else
                        sm_msg "`eval_gettext \"Could not create resources for the resource group \\\"\\\${RGName}\\\". Contact Software Support.\"`"
                    fi
                else
                    sm_msg "`eval_gettext \"Could not configure the resource group \\\"\\\${RGName}\\\" on this node. Contact Software Support.\"`"
                fi
            else
                sm_msg "`eval_gettext \"Could not configure the resource group \\\"\\\${RGName}\\\" on the peer node. Contact Software Support.\"`"
            fi

            shift
        done

        if (( OKcount > 0 )); then
            log_command_success
            exit 0
        else
            log_command_failed
            exit 1
        fi
    fi
}

#####################################################
# Process the enable-rg parameter
#####################################################
proc_enable_rg () {
    local OKcount=0
    local RC
    local RGNames
    local RGName
    local OwnerAttrVal
    local cmd_out

    if [ -n "$enable_rg" ]; then
        debug "smha --enable-rg $enable_rg called"

    	chk_ha_setup

        RGNames=${enable_rg//,/ }     # replace commas by spaces
        set $RGNames

        while [ -n "$1" ]; do
            RGName=$1
            debug "Enable subystem specified with resource group name $RGName"

            # check if this is a "switchable" resource group, i.e. the Owner attribute value is not NULL

            OwnerAttrVal=$(LC_ALL=C lsrsrc-api -s "IBM.ResourceGroup::Name == \"${RGName}\"::Owner" 2>&1)
            RC=$?

            if [ $RC -eq 0 ]; then
                if [ -n "$OwnerAttrVal" ]; then

                    # it is switchable; change the Owner value to Enable

                    cmd_out=$(LC_ALL=C chrsrc-api -s "IBM.ResourceGroup::Name == \"${RGName}\"::Owner::Enabled" 2>&1)
                    RC=$?

                    if [ $RC -eq 0 ]; then
                        ch_rg_online $RGName
                        sm_msg "`eval_gettext \"The resource group \\\"\\\${RGName}\\\" has been enabled.\"`"
                        (( OKcount++ ))

                    else
                        sm_msg "`eval_gettext \"The resource group \\\"\\\${RGName}\\\" cannot be enabled. Contact Software Support.\"`"
                        debug "chrsrc-api returned: $cmd_out"
                    fi
                else
                    sm_msg "`eval_gettext \"The resource group \\\"\\\${RGName}\\\" is not a user controllable resource group. Specify a controllable resource group and try again.\"`"
                fi

            elif [ $RC -eq 6 ]; then
                sm_msg "`eval_gettext \"The resource group \\\"\\\${RGName}\\\" is not defined. Specify a defined, controllable resource group and try again.\"`"

            else
                sm_msg "`eval_gettext "The high availability environment encounted an error. Contact Software Support."`"
                debug "lsrsrc-api returned: $OwnerAttrVal"
            fi

            shift
        done

        if (( OKcount > 0 )); then
            log_command_success
            exit 0
        else
            log_command_failed
            exit 1
        fi
    fi
}

#####################################################
# Process the disable-rg parameter
#####################################################
proc_disable_rg () {
    local OKcount=0
    local RC
    local RGNames
    local RGName
    local OwnerAttrVal
    local cmd_out

    if [ -n "$disable_rg" ]; then
        debug "smha --disable-rg $disable_rg called"

    	chk_ha_setup

        RGNames=${disable_rg//,/ }     # replace commas by spaces
        set $RGNames

        while [ -n "$1" ]; do
            RGName=$1
            debug "Disable subystem specified with resource group name $RGName"

            # check if this is a "switchable" resource group, i.e. the Owner attribute value is not NULL

            OwnerAttrVal=$(LC_ALL=C lsrsrc-api -s "IBM.ResourceGroup::Name == \"${RGName}\"::Owner" 2>&1)
            RC=$?

            if [ $RC -eq 0 ]; then
                if [ -n "$OwnerAttrVal" ]; then

                    # it is switchable; change the Owner value to Disable

                    cmd_out=$(LC_ALL=C chrsrc-api -s "IBM.ResourceGroup::Name == \"${RGName}\"::Owner::Disabled" 2>&1)
                    RC=$?

                    if [ $RC -eq 0 ]; then
                        take_rg_offline $RGName
                        sm_msg "`eval_gettext \"The resource group \\\"\\\${RGName}\\\" has been disabled.\"`"
                        (( OKcount++ ))

                    else
                        sm_msg "`eval_gettext \"The resource group \\\"\\\${RGName}\\\" cannot be disabled. Contact Software Support.\"`"
                        debug "chrsrc-api returned: $cmd_out"
                    fi
                else
                    sm_msg "`eval_gettext \"The resource group \\\"\\\${RGName}\\\" is not a user controllable resource group. Specify a controllable resource group and try again.\"`"
                fi

            elif [ $RC -eq 6 ]; then
                sm_msg "`eval_gettext \"The resource group \\\"\\\${RGName}\\\" is not defined. Specify a defined, controllable resource group and try again.\"`"

            else
                sm_msg "`eval_gettext "The high availability environment encounted an error. Contact Software Support."`"
                debug "lsrsrc-api returned: $OwnerAttrVal"
            fi

            shift
        done

        if (( OKcount > 0 )); then
            log_command_success
            exit 0
        else
            log_command_failed
            exit 1
        fi
    fi
}

SMHAP_ACQUIRED_LOCK=no
trap_on_exit () {
    if [ "$SMHAP_ACQUIRED_LOCK" = "yes" ]; then
        rw_lock write free CmdSMHA $PROGNAME
    fi
}

###################################
## Main code                     ##
###################################

if [ $# -eq 0 ]; then
    echo_help
    exit 0
fi

ORIG_PARMS="$*"
LOGCOMMAND=ON

proc_parms $*

##
## There are only a few ways this command can be called.
## Run down through each and process them.
##

# Take care of help first
proc_help

# get a write lock to serialize calls to this script and others that may be changing
# the state of the HA resources

if rw_lock write get CmdSMHA $PROGNAME 3
then
    SMHAP_ACQUIRED_LOCK=yes
    trap trap_on_exit EXIT

    if files_exist $HACMD_INPROGRESS ; then
        sm_msg "`eval_gettext "The high availability environment is busy. Wait a few minutes, then try this command again."`"
        exit 1
    fi
else
    sm_msg "`eval_gettext "The high availability environment is busy. Wait a few minutes, then try this command again."`"
    exit 1
fi

# If start is specified
proc_start
# If stop is specified
proc_stop
# If suspend is specified
proc_suspend
# If resume is specified
proc_resume
# If reset is specified
proc_reset
# If the validate parameter is specified with the failover parm
proc_validate
# If the cancel parameter is specified with the failover parm
proc_cancel
# If forceActive is specified
proc_forceActive
# If configure-rg is specified
proc_configure_rg
# If enable-rg is specified
proc_enable_rg
# If disable-rg is specified
proc_disable_rg
# If cleanup is specified
proc_cleanup
# If failover is specified
proc_failover

log_command_success

exit 0
