#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
# 
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996
#

PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################
load_common_code $INSTALL_DIR/ha/bin/include/smhadrbd.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadir.sh


RSCT_BKUP=/var/ct.backup
UPGRADE_LOG_FILE=/opt/ibm/director/vmi/upgrade/save/log/upgrade_deferred.log

hdl_upgrade () {
    
   	# Once we restore RSCT and TSA configuration, TSA will start checking 
   	# the status on all the resources that are defined in the configuration that
   	# we are about to restore.
   	# Some of these resources start out as services that by default would be started
   	# by the operating system (via /etc/inittab or via the chkconfig framework).
   	# Since some of these services are installed as "on" by default, they are 
   	# probably running right now.  We need to turn them off.
   	# run through the scripts in the plugins/conf directory and pass them an 
   	# "upgrade" parm, so they can turn off any service that shipped as "on" by
   	# default.
        does_ha_config_exist
        if [ $? -eq 1 ]; then
            # This will run on both active and passive nodes in a "coordinated upgrade".  We need
            # to remove the nonstop daemon trigger file because it is placed on the spare in proddata.
            echo -e "$PROGNAME: HA is configured: removing nonstop daemon trigger file" >>$UPGRADE_LOG_FILE 2>&1
            debug "Removing non-stop daemon trigger file"
            deactivate_dir_server_nonstop_non_mirrored_files 

            echo -e "$PROGNAME: HA is configured: call plugins with upgrade parm" >>$UPGRADE_LOG_FILE 2>&1
            debug "HA is configured: call plugins with upgrade parm"
   	    /opt/ibm/director/ha/bin/priv/smhacplug.sh conf upgrade

            # Reset the dirc settings to passive, they'll get set to active if director is started.
            dirc_use_settings passive

        else
           echo -e "$PROGNAME: HA is not configured: NOT calling plugins with upgrade parm" >>$UPGRADE_LOG_FILE 2>&1
           debug "HA is not configured: NOT calling plugins with upgrade parm"
        fi
    
    rm -f /var/ct/cfg/ctrmc.NoStart

    if [ -e $RSCT_BKUP ]; then
    
        # Stop rsct
        /usr/bin/rmcctrl -z
        
        debug "ctrestore start"
        local rst_out=`/usr/sbin/rsct/bin/ctrestore 2>&1`
        rst_rc=$?
        debug "ctrestore rc:$rst_rc output:$rst_out"
        
        if [ $rst_rc -ne 0 ]; then
            echo -e "$PROGNAME: ctrestore failed" >>$UPGRADE_LOG_FILE 2>&1
            exit $rst_rc
        fi
        rm -rf $RSCT_BKUP
        
        # Startup rsct
        #/usr/bin/rmcctrl -s >/dev/null 2>&1
    fi
    
    # It takes a while for RSCT configuration to kick in
    sleep 60
    
}

###################################
## Main code                     ##
###################################
debug "Running deferred upgrade script"

hdl_upgrade

exit 0
