#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996
#

PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################

####### NOTE NOTE NOTE NOTE ######
# Any common routines that you run from this procedure will come from
# the "old release" via /opt.  They do NOT come from /spare so don't
# get yourself confused!!!!
##################################

load_common_code $INSTALL_DIR/ha/bin/include/smhadrbd.sh

DEST_PRE=/spare
DEST_MIRROR0=$DEST_PRE/mirror0
DEST_OPT=$DEST_PRE/opt
DEST_VAR=$DEST_PRE/var
DEST_HOME=$DEST_PRE/home
DEST_ROOT=$DEST_PRE

DRBD_SOURCE=/etc/drbd.conf

RSCT_BKUP=/var/ct.backup

cpy_ha_config () {
	debug "cp -a $SMHA_CONFIGDIR ${DEST_ROOT}${SMHA_DIR}"
	cp -a $SMHA_CONFIGDIR ${DEST_ROOT}${SMHA_DIR}
	
	# Remove unwanted files
	rm -f ${DEST_ROOT}$MD_FILE
	rm -f ${DEST_ROOT}${SMHA_PAUSE_FILE}-*
        rm -f ${DEST_ROOT}${DRBD_NOSTART_FLAG}
	rm -rf ${DEST_ROOT}$SMHA_CONFIGDIR/.mutex*
	
}

cpy_ha_MQ_mirrored_dirs () {
        # this declare should be in smhacommon.sh, but since it will be used from 
        # this script during upgrade from isdvmi12b and run from /spare,
        # I can't move it to smhacommon.sh until the next isdvmi release.
        # This is because when it is run, an include from "/" would not find it...
        HA_MQ_MIRRORED_DIRS="$DIR_OVERRIDES_DIR $LWI_SECURITY_DIR $DIR_SECURITY_DIR $DIR_AMCERTS_DIR"
        
        # Loop thru the MQ mirrored dirs, and make sure that they get copied to /spare.  This will be a redundant
        # copy when run on the Active node since mirrored storage already is copied.  But, on the passive node, the
        # CDSDA scripts do not copy mirrored storage.  Since we try to maintain THREE copies of these directories, so that
        # the standby GUI can run in the LWI container when the mirrored storage is not mounted, we need to copy the
        # directories even on the passive node during an upgrade.
        for dir in $HA_MQ_MIRRORED_DIRS
        do
           # in case the directory doesn't exist on /spare yet.  Really no need to check/log rc from mkdir.
           debug "mkdir -p /spare$dir" 
           mkdir -p /spare$dir
           current_files=`ls $dir`
           debug "files in $dir that will be copied to /spare/$dir are:\n$current_files"
           # wanna copy from $dir=/opt/blah to $target_parent_dir=/spare/opt, so need to strip off last dir of path 
           # for the target
           target_parent_dir=/spare${dir%/*}
           debug "cp -a $dir $target_parent_dir"
           cp -a $dir $target_parent_dir
           rc=$?
           debug "cp -a $dir $target_parent_dir returned rc=$rc"
        done
}

cpy_ha_persist () {
    debug "cp -a $SMHA_DIR/persist $DEST_ROOT/$SMHA_DIR"
    nice -n 10 cp -a $SMHA_DIR/persist $DEST_ROOT/$SMHA_DIR
        
}

cpy_ha_logs () {
    debug "cp -a $LOG_DIR $DEST_ROOT/$SMHA_DIR"
    nice -n 10 cp -a $LOG_DIR ${DEST_ROOT}${SMHA_DIR}
        
}

#cpy_tiv_guid () {
	# Not sure we need this since VMI copies /etc/ files....
#    if [ -f /etc/ibm/tivoli/AM.guid ]; then
#        debug "cp -a /etc/ibm/tivoli/AM.guid $DEST_ROOT/etc/ibm/tivoli/AM.guid"
#        cp -a /etc/ibm/tivoli/AM.guid $DEST_ROOT/etc/ibm/tivoli/AM.guid
#    fi
#        
#}

remove_ldap_symlink () {
  local FOUND_TEMP_SYMLINKS
  FOUND_TEMP_SYMLINKS=`find /spare/etc/init.d/rc?.d/???ldap 2> /dev/null`
  if [ $? -eq 0 ]; then
     local ffile
     for ffile in $FOUND_TEMP_SYMLINKS
     do
        rm -f $ffile
     done
  fi
  
}

cpy_drbd_info () {
    local i
    local j
    local sed_line=
    local TMPFILE=/tmp/smhaupgrd.$$

    debug "cp drbd info"

    on_host=$HOST

    $GETMIRROR_CMD > $TMPFILE

    i=0
    while read device mirror
    do
        devices[$i]=$device
        mirrors[$i]=$mirror
        i=$(( i + 1 ))
    done < $TMPFILE

    debug "devices => ${devices[*]}"
    debug "mirrors => ${mirrors[*]}"

    $GETMIRROR_CMD -s > $TMPFILE

    while read device mirror
    do
        debug "device=$device  mirror=$mirror"

        j=0
        for (( j=0; j < i; j++ ))
        do
            if [ "${mirrors[$j]}" = "$mirror" ]
            then
                if [ "${devices[$j]}" != "$device" ]
                then
                    debug "drbd.conf conversion; from ${devices[$j]} to $device"

                    sed_line_section="s#disk ${devices[$j]}#disk ${device}#g"

                    if [ -z "$sed_line" ]; then
                        sed_line="$sed_line_section"
                    else
                        sed_line="${sed_line};${sed_line_section}"
                    fi
                fi
                break
            fi
        done
    done < $TMPFILE
    rm -f $TMPFILE

    debug "drbd sed line is: $sed_line"
    sed "/on ${on_host}/ { N
    N
    N
    N
    N
    $sed_line}" /etc/drbd.conf >${DEST_ROOT}/etc/drbd.conf

    cp -a /etc/drbd.conf $DEST_ROOT/etc/drbd.conf.spare
}


bk_rsct () {
	
	if [ ! -d $RSCT_BKUP ]; then
        bkup_out=`nice -n 10 /usr/sbin/rsct/bin/ctbackup 2>&1` 
        # Remove the old copy
        rm -rf ${DEST_ROOT}${RSCT_BKUP}
        # Move the new copy to the spare
        nice -n 10 mv $RSCT_BKUP $DEST_VAR
        debug "RSCT BKUP: $bkup_out"
	fi
}

bk_ha_ssl_keys () {

    if [ -e /root/.ssh/authorized_keys2 ]; then
        cp -a /root/.ssh/authorized_keys2 /spare/root/.ssh/
    fi
}

###################################
## Main code                     ##
###################################
HA_DO_STOP=0
RC=0
debug " enter"

bk_rsct
#disallow_rsct_start   # prevent RSCT from staring on reboot, until ctrestore is called
touch /spare/var/ct/cfg/ctrmc.NoStart
bk_ha_ssl_keys
#cpy_tiv_guid

is_configured
if [ $? -eq 0 ]; then
    debug "HA is not configured."
else
    # before we get into all the active vs passive stuff on upgrade, we need to hack in a fix to 
    # clean up the ldap server.  /etc/openldap/ is a mirrored directory, and as such, nothing was
    # copied into it via the upgrade scripts on the way down.  On the way back up during the reboot,
    # the runlevel 3 symlinks will be there by default since it is shipped that way.  
    # The upgrade_deferred code path on the way back up has a hook to call all of the "ha/plugins/conf" scripts
    # with "conf" "upgrade", which would remove the symlinks, but this doesn't work for the ldap server case,
    # because the /etc/init.d/ldap script has a grep/find/awk command invocation that hangs because the 
    # /etc/openldap/slapd.d/ directory is empty.  This causes the IPL to hang on the way back up (never gets
    # to the updf line in /etc/inittab).  To fix this problem, this late in the release, we have decided to 
    # just remove the symlinks for ldap from runlevel 3, while /spare is still mounted.  
    remove_ldap_symlink
    
   	# Originally the code was creating the .drbd.upgrade flag file
   	# in smhaupgrd_def.sh (which is run after reboot on the way back up).
   	# That would cause the flag file to get created on both active and passive
   	# nodes (remember that for GA1 you only upgraded active and you reinstalled
   	# the passive).  The flag file is used by the smhatsadrbd.sh script to determine
   	# which way the replication should happen.  We ONLY want it on the 
   	# active node, so moving it here.
   	is_passive
   	rc=$?
   	debug "is_passive returned $rc"
   	if [ $rc -eq 0 ]; then
   		# active node.  create the flag file in /opt.../ha/config dir and
   		# let it get copied to /spare with the rest.
   		debug "creating $DRBD_UPGRADE_FLAG"
   		touch $DRBD_UPGRADE_FLAG

        HA_DO_STOP=1
   	fi
    create_stop_maint_flagfile   # keep smhamaint.sh from running on next boot
   	
    ## Run through the items to copy to the spare disk    
    cpy_ha_config
    cpy_ha_MQ_mirrored_dirs
    cpy_ha_persist
    cpy_drbd_info

fi

if [ $HA_DO_STOP -eq 1 ]; then
    # set the "DRBD No Start" flag on the Passive node, then stop the
    # DRBD base resource on the passive node. TSA will then attempt
    # to restart it, but, as a result of the flag, the smhatsadrbd.sh
    # script will fail the start command. TSA will then set the DRBD
    # base resource "failed offline".
    #
    # N.B. we take down the DRBD base resource on the peer node to
    # avoid having DRBD communicating
    # with its peer, since after the reboot of this (Active) node,
    # the local mirrors are Inconsistent while the peer's mirrors
    # are likely to be UpToDate. Once the peer is upgraded and
    # rebooted, its mirrors will also be Inconsistent and the data
    # will flow from the local mirrors to the peer. After the reboot
    # of this node the mirrors will be made Primary.
    #
    # After the upgrade of the passive node, the failed offline
    # DRBD base resources is reset.

    # set flag on peer node
    ${CMD_DIR}/smhapause.sh -p drbdNoStart
    sleep 5

    get_remote_rsct_node_name
    stop_out=$(LC_ALL=C stoprsrc -s "Name  == \"drbdb\" && NodeNameList == \"${REMOTE_RSCT_NODE_NAME}\"" IBM.Application 2>&1)
    RC=$?
    if [ $RC -eq 0 ]; then

        sleep 5             # give TSA some time to stop the resource and attempt to restart it

        done_sleeping=0
        # wait for up to 2 minute
        for i in {1..24}
        do  
            cmd_out=$(LC_ALL=C lsrsrc-api -s "IBM.Application::Name  == \"drbdb\" && NodeNameList == \"${REMOTE_RSCT_NODE_NAME}\"::OpState")
            RC=$?   
            if [ $RC -eq 0 ]; then 
                # command worked 
                if [ "$cmd_out" == "${TSA_FAILED_OFFLINE}" ]; then
                    debug "detected peer node ${REMOTE_RSCT_NODE_NAME} drdb base resource failed offline"
                    done_sleeping=1 
                    break           
                else        
                    debug "sleep number $i for 5 seconds, desired_state=${TSA_FAILED_OFFLINE}, actual_state=$cmd_out"
                    sleep 5        
                fi          
            else    
                debug "lsrsrc-api failed with RC=$RC"
                sleep 5    
            fi      
        done

        if [ $done_sleeping -eq 0 ]; then
            debug "Could not get remote drbdb to fail offline"
            RC=1
        fi
    else
        debug "Could not stop peer drbdb, RC=$RC out: ${stop_out}"
    fi
fi

# Copy the logs last so we have the most recent info
cpy_ha_logs

# work around a CSDA bug where /etc/services ends up having
# 600 permissions on the passive node, so DB2 won't start
/bin/chmod 644 /spare/etc/services

debug " exit; RC=$RC"
exit $RC
