#!/bin/bash
# Licensed Materials - Property of IBM
# 
# 09N6996
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)
#### Use a special path for TSA monitoring scripts
PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/sbin/rsct/bin
export PATH

EXPLOITER_COMMON_CODE=/opt/ibm/director/ha/bin/include/smha_exploiter_common.sh

if [ -f $EXPLOITER_COMMON_CODE ]; then
    source $EXPLOITER_COMMON_CODE
else
    echo "$PROGNAME:Exploiter common code not found."
    exit 1
fi
#############################################################

RC=0

Action=${1}

# This sample start/stop/monitor command script can be used for more than one resource.
# A typical exploiter would only be managing a single resource, so there is no need to
# configure a second argument to this command in that case.

Resource=${2}
if [ "$Resource" = "Enabled" ]; then
    ResourceName="samplecontrolledE"
elif [ "$Resource" = "Disabled" ]; then
    ResourceName="samplecontrolledD"
else
    ResourceName="samplecontrolled"
fi

# A start and stop command must exit with 0 on success and not 0 on failure. The monitor
# command must exit with a value that indicates the status of the underlying appliance
# resource. The possible values are defined by smha_exploiter_common.sh and begin with the
# TSA_ prefix. Only these values may be used. The most common values used are
#
#   TSA_UNKNOWN=0           Status cannot be determined or there is an error
#   TSA_ONLINE=1            The resource is online, i.e. active
#   TSA_OFFLINE=2           The resource is offline, i.e. inactive
#   TSA_PENDING_ONLINE=5    The resource is starting, but not yet active
#   TSA_PENDING_OFFLINE=6   The resource is stopping, but not yet inactive
#
# In TSA terminology, each of these status values is called an OpState (Operational State).
#
# Please refer to the additional documentation in the samplecontrolledD-rg.sh or
# samplecontrolledE-rg.sh sample scripts.

# Note the calls to the smha_record_tsa_action function. Each call logs an entry
# in the HA log file. There are HA tools that examine entries for debugging.
# An actual script must also have these calls, in the place and manner shown
# below. The first two arguments to the function are always the Action and
# ResourceName. In the case of start/stop case entries, the third argument is "-". 
# In all cases the final call is passed the exit code as the third argument. Note
# that status only logs one entry.

case ${Action} in
start)
    smha_record_tsa_action $Action ${ResourceName} "-"

    # Normally, you would call your startup process here
    # e.g. /etc/init.d/mydaemon start. For this example, a file
    # is created indicating the resource is online.

    smha_debug_tsa "${ResourceName} program started"

    touch $PROGPATH/${ResourceName}_online
    ;;
stop)
    smha_record_tsa_action $Action ${ResourceName} "-"

    # Normally, you would call your stop process here
    # e.g. /etc/init.d/mydaemon stop. For this example, a file
    # that indicated the resource is online is removed.

    # you may want to perform some sanity checking here to make sure
    # the process really died.  Especially in the case if the process
    # contains files in shared storage which needs to be unmounted
    # in the case of a manual failover.
    
    smha_debug_tsa "${ResourceName} program stopped"
    if [ -f $PROGPATH/${ResourceName}_online ]; then
        rm $PROGPATH/${ResourceName}_online
    fi
    ;;

status)
    # Normally, you would call your status process here
    # e.g. /etc/init.d/mydaemon status
    # statusrc=$?

    if [ -f $PROGPATH/${ResourceName}_online ]; then
        RC=${TSA_ONLINE}
    else
        RC=${TSA_OFFLINE}
    fi
    
    ;;
esac
#debug_tsa "$PROGNAME $Action: rc=$RC"
smha_record_tsa_action $Action $ResourceName $RC
exit $RC
